@@ -32,15 +32,16 @@
 #include "logger/LoggerFactory.h"
 #include "pool/Pools.h"
 #include "util/Utils.h"
+#include "Properties.hpp"
 #include "jdbccompat.hpp"
 #include "ExceptionFactory.h"
+#include "pool/Pool.h"
 
 namespace sql
 {
 namespace mariadb
 {
-  Shared::Logger MariaDbConnection::logger= LoggerFactory::getLogger(typeid(MariaDbConnection));
-
+  Logger* MariaDbConnection::logger= LoggerFactory::getLogger(typeid(MariaDbConnection));
   /**
     * Creates a new connection with a given protocol and query factory.
     *
@@ -48,13 +49,12 @@ namespace mariadb
     */
   MariaDbConnection::MariaDbConnection(Shared::Protocol& _protocol) :
     protocol(_protocol),
-    options(protocol->getOptions()),
-    exceptionFactory(ExceptionFactory::of(this->getServerThreadId(), options)),
     lock(_protocol->getLock()),
-    pooledConnection(nullptr),
-    nullCatalogMeansCurrent(options->nullCatalogMeansCurrent),
+    options(protocol->getOptions()),
     _canUseServerTimeout(protocol->versionGreaterOrEqual(10, 1, 2)),
-    sessionStateAware(protocol->sessionStateAware())
+    sessionStateAware(protocol->sessionStateAware()),
+    nullCatalogMeansCurrent(options->nullCatalogMeansCurrent),
+    exceptionFactory(ExceptionFactory::of(this->getServerThreadId(), options))
   {
     if (options->cacheCallableStmts)
     {
@@ -70,11 +70,11 @@ namespace mariadb
     * @return connection object
     * @throws SQLException if any connection error occur
     */
-  MariaDbConnection* MariaDbConnection::newConnection(UrlParser &urlParser, GlobalStateInfo *globalInfo)
+  MariaDbConnection* MariaDbConnection::newConnection(Shared::UrlParser &urlParser, GlobalStateInfo *globalInfo)
   {
-    if (urlParser.getOptions()->pool)
+    if (urlParser->getOptions()->pool)
     {
-      //return Pools::retrievePool(urlParser)->getConnection();
+      return dynamic_cast<MariaDbConnection*>(Pools::retrievePool(urlParser)->getPoolConnection()->getConnection());
     }
     Shared::Protocol protocol(Utils::retrieveProxy(urlParser, globalInfo));
 
@@ -97,7 +97,15 @@ namespace mariadb
 
   MariaDbConnection::~MariaDbConnection()
   {
-    protocol->closeExplicit();
+    if (poolConnection == nullptr && !returnedToPool) {
+      protocol->closeExplicit();
+    }
+    else {
+      if (!isClosed()) {
+        this->poolConnection->returnToPool();
+        // close() would also marked connection as closed, but since this object is about to die, it does not need that any more.
+      }
+    }
   }
 
 
@@ -105,14 +113,20 @@ namespace mariadb
     return protocol;
   }
 
+  void MariaDbConnection::setPoolConnection(MariaDbPoolConnection* _poolConnection)
+  {
+    poolConnection= _poolConnection;
+  }
+
   /**
-    * creates a new statement.
-    *
-    * @return a statement
-    * @throws SQLException if we cannot create the statement.
-    */
+  * creates a new statement.
+  *
+  * @return a statement
+  * @throws SQLException if we cannot create the statement.
+  */
   Statement* MariaDbConnection::createStatement()
   {
+    logger->trace("Creating Stmt on connection:", std::hex, this->poolConnection, "Protocol:", protocol.get(), "expClosed:", protocol->isExplicitClosed());
     checkConnection();
     return new MariaDbStatement(this, ResultSet::TYPE_FORWARD_ONLY, ResultSet::CONCUR_READ_ONLY, exceptionFactory);
   }
@@ -388,7 +402,18 @@ namespace mariadb
     {
     case 's':
     case 'S':
-      return std::tolower(*++it) == 'e' && std::tolower(*++it) == 'l' && std::tolower(*++it) == 'e' && std::tolower(*++it) == 'c' && std::tolower(*++it) == 't';
+    {
+      switch (*++it) {
+      case 'e':
+      case 'E':
+        return std::tolower(*++it) == 'l' && std::tolower(*++it) == 'e' && std::tolower(*++it) == 'c' && std::tolower(*++it) == 't';
+      case 'h':
+      case 'H':
+        return std::tolower(*++it) == 'o' && std::tolower(*++it) == 'w';
+      default:
+        return false;
+      }
+    }
     case 'u':
     case 'U':
       return std::tolower(*++it) == 'p' && std::tolower(*++it) == 'd' && std::tolower(*++it) == 'a' && std::tolower(*++it) == 't' && std::tolower(*++it) == 'e';
@@ -397,7 +422,8 @@ namespace mariadb
       return std::tolower(*++it) == 'n' && std::tolower(*++it) == 's' && std::tolower(*++it) == 'e' && std::tolower(*++it) == 'r' && std::tolower(*++it) == 't';
     case 'd':
     case 'D':
-      return std::tolower(*++it) == 'e' && std::tolower(*++it) == 'l' && std::tolower(*++it) == 'e' && std::tolower(*++it) == 't' && std::tolower(*++it) == 'e';
+      return std::tolower(*++it) == 'o'/*DO*/ || /*or delete*/(std::tolower(*it) == 'e' && std::tolower(*++it) == 'l' && std::tolower(*++it) == 'e' &&
+                                                               std::tolower(*++it) == 't' && std::tolower(*++it) == 'e');
     case 'r':
     case 'R':
       return std::tolower(*++it) == 'e' && std::tolower(*++it) == 'p' && std::tolower(*++it) == 'l' && std::tolower(*++it) == 'a' && std::tolower(*++it) == 'c' &&
@@ -435,7 +461,8 @@ namespace mariadb
   {
     if (!sql.empty())
     {
-      SQLString sqlQuery= Utils::nativeSql(sql, protocol.get());
+      SQLString buffer;
+      const SQLString& sqlQuery= Utils::nativeSql(sql, buffer, protocol.get());
 
       if (options->useServerPrepStmts && shouldPrepareOnServer(sql))
       {
@@ -444,7 +471,6 @@ namespace mariadb
         //try {
         return new ServerSidePreparedStatement(this, sqlQuery, resultSetScrollType, resultSetConcurrency,
             autoGeneratedKeys, exceptionFactory);
-        //!!!! TODO: !!!!! Need to fix exceptions, so we do not throw SQLException's only
         /*}
         catch (SQLNonTransientConnectionException& e) {
           throw e;
@@ -526,8 +552,7 @@ namespace mariadb
         
       }
       if (!wrongFormat) {
-        native= Utils::nativeSql(sql, protocol.get());
-        query= &native;
+        query= &Utils::nativeSql(sql, native, protocol.get());
         firstUsefulChar= Utils::skipCommentsAndBlanks(StringImp::get(native));
       }
     }
@@ -679,7 +704,8 @@ namespace mariadb
 
   SQLString MariaDbConnection::nativeSQL(const SQLString& sql)
   {
-    return Utils::nativeSql(sql, protocol.get());
+    SQLString buffer;
+    return Utils::nativeSql(sql, buffer, protocol.get());
   }
 
   /**
@@ -710,7 +736,7 @@ namespace mariadb
     if (stmt)
     {
       stateFlag|= ConnectionState::STATE_AUTOCOMMIT;
-      stmt->executeUpdate(SQLString("set autocommit=").append((autoCommit) ? '1' : '0'));
+      stmt->executeUpdate(SQLString("SET AUTOCOMMIT=").append((autoCommit) ? '1' : '0'));
     }
   }
 
@@ -780,13 +806,27 @@ namespace mariadb
     */
   void MariaDbConnection::close()
   {
-    if (pooledConnection)
+    if (poolConnection)
     {
-      rollback();
-      pooledConnection->fireConnectionClosed();
+      rollback(); //TODO: should it be here?
+      logger->trace("Closing:", std::hex, this->poolConnection, "Protocol:", protocol.get(), "expClosed:", protocol->isExplicitClosed());
+      // Should do this before returning to the pool, or we can do this when other connection object has already seized this physical connection
+      this->markClosed(true);
+      // We returning connectin to the pool where it can be re-used by other connection, which will make it appear as "not closed" again.
+
+      this->poolConnection->returnToPool();
+      // if we don't reset protocol here, and current object still exist, connection may become used by more than one conneciton object
+      // But have to do it after returning to the pool, as it is used for creation of the new object
+      protocol= nullptr;
+      poolConnection= nullptr;
+      returnedToPool= true;
+      
+      
       return;
     }
-    protocol->closeExplicit();
+    else if (!returnedToPool){
+      protocol->closeExplicit();
+    }
   }
 
   /**
@@ -796,7 +836,7 @@ namespace mariadb
     */
   bool MariaDbConnection::isClosed()
   {
-    return protocol->isClosed();
+    return (!protocol || protocol->isClosed() || returnedToPool);
   }
 
   /**
@@ -963,7 +1003,7 @@ namespace mariadb
     */
   void MariaDbConnection::setTransactionIsolation(int32_t level) {
     try {
-      stateFlag |=ConnectionState::STATE_TRANSACTION_ISOLATION;
+      stateFlag|= ConnectionState::STATE_TRANSACTION_ISOLATION;
       protocol->setTransactionIsolation(level);
     }
     catch (SQLException& e) {
@@ -1202,7 +1242,7 @@ namespace mariadb
       return false;
     }
     try {
-      return protocol->isValid(timeout *1000);
+      return protocol->isValid(timeout*1000);
     }
     catch (SQLException&) {
       // eat
@@ -1240,7 +1280,7 @@ namespace mariadb
         {
           std::map<SQLString, ClientInfoStatus>failures;
           failures.insert({ name, ClientInfoStatus::_REASON_UNKNOWN });
-          throw SQLException("ClientInfoException: Connection closed");// SQLClientInfoException("Connection* closed", failures, sqle);
+          throw SQLException("ClientInfoException: Connection closed");// SQLClientInfoException("Connection closed", failures, sqle);
         }
       }
       else {
@@ -1642,12 +1682,12 @@ namespace mariadb
     * @param executor executor
     * @throws SQLException if security manager doesn't permit it.
     */
-  void MariaDbConnection::abort(sql::Executor* /*executor*/)
+  void MariaDbConnection::abort(sql::Executor* executor)
   {
     if (this->isClosed()) {
       return;
     }
-    throw SQLFeatureNotSupportedException("Connection abort is not supported yet");
+    //throw SQLFeatureNotSupportedException("Connection abort is not supported yet");
 
 #ifdef JDBC_SPECIFIC_TYPES_IMPLEMENTED
     SQLPermission sqlPermission= new SQLPermission("callAbort");
@@ -1655,11 +1695,16 @@ namespace mariadb
     if (securityManager != nullptr) {
       securityManager.checkPermission(sqlPermission);
     }
-    if (executor == nullptr) {
-      throw ExceptionMapper.getSqlException("Cannot abort the connection: NULL executor passed");
-    }
-    executor.execute(protocol->abort());
 #endif
+#ifdef WE_STILL_WANT_TO_ABORT_IN_SEPARATE_THREAD
+    // There is no need to throw here as we do not use it anyway
+    if (executor == nullptr) {
+      ExceptionFactory::INSTANCE.create("Cannot abort the connection: NULL executor passed"); // this will throw
+    }
+    /* There is no sense to abort it in the separate thread, since Pool destructs connection after this call, and this has to be syncronized anyway */
+    executor->execute(std::bind(&Protocol::abort, &*protocol));
+#endif
+    protocol->abort();
   }
 
   /**
@@ -1683,7 +1728,7 @@ namespace mariadb
       throw SQLException("The catalog name may not be empty", "XAE05");
     }
     try {
-      stateFlag |= ConnectionState::STATE_DATABASE;
+      stateFlag|= ConnectionState::STATE_DATABASE;
       protocol->setCatalog(schema);
     }
     catch (SQLException &e) {
@@ -1698,32 +1743,14 @@ namespace mariadb
     * @param milliseconds network timeout in milliseconds.
     * @throws SQLException if security manager doesn't permit it.
     */
-  void MariaDbConnection::setNetworkTimeout(Executor* /*executor*/, int32_t /*milliseconds*/)
+  void MariaDbConnection::setNetworkTimeout(Executor* executor, uint32_t milliseconds)
   {
-    throw SQLFeatureNotImplementedException("setNetworkTimeout is not yet implemented");
-
-#ifdef JDBC_SPECIFIC_TYPES_IMPLEMENTED
     if (this->isClosed()) {
       throw SQLException("Connection::setNetworkTimeout cannot be called on a closed connection");//ExceptionMapper.getSqlException
     }
 
-    if (milliseconds <0) {
-      throw ExceptionMapper.getSqlException(
-        "Connection::setNetworkTimeout cannot be called with a negative timeout");
-    }
-    SQLPermission sqlPermission= new SQLPermission("setNetworkTimeout");
-    SecurityManager securityManager= System.getSecurityManager();
-    if (securityManager != nullptr) {
-      securityManager.checkPermission(sqlPermission);
-    }
-    try {
-      stateFlag|= ConnectionState::STATE_NETWORK_TIMEOUT;
-      protocol->setTimeout(milliseconds);
-    }
-    catch (SocketException& se) {
-      throw ExceptionMapper.getSqlException("Cannot set the network timeout", se);
-    }
-#endif
+    stateFlag|= static_cast<int32_t>(ConnectionState::STATE_NETWORK_TIMEOUT);
+    protocol->setTimeout(milliseconds);
   }
 
 
@@ -1755,7 +1782,7 @@ namespace mariadb
     bool useComReset=
       options->useResetConnection
       && ((protocol->isServerMariaDb() && protocol->versionGreaterOrEqual(10, 2, 4))
-        || (!protocol->isServerMariaDb() && protocol->versionGreaterOrEqual(5, 7, 3)));
+       || (!protocol->isServerMariaDb() && protocol->versionGreaterOrEqual(5, 7, 3)));
     if (useComReset) {
       protocol->reset();
     }
@@ -1815,5 +1842,11 @@ namespace mariadb
 
     return new CallableParameterMetaData(preparedStatement->executeQuery(), isFunction);
   }
+
+
+  void MariaDbConnection::markClosed(bool closed)
+  {
+    protocol->markClosed(closed);
+  }
 }
 }
