@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2020,2023 MariaDB Corporation AB
+   Copyright (C) 2020,2024 MariaDB Corporation AB
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -24,6 +24,7 @@
 #include "ExceptionFactory.h"
 #include "util/Utils.h"
 #include "Results.h"
+#include "pool/ConnectionEventListener.h"
 
 namespace sql
 {
@@ -41,7 +42,7 @@ namespace mariadb
     {"\\",     "\\\\"},
   };
 
-  Shared::Logger MariaDbStatement::logger= LoggerFactory::getLogger(typeid(MariaDbStatement));
+  Logger* MariaDbStatement::logger= LoggerFactory::getLogger(typeid(MariaDbStatement));
   /**
    * Creates a new Statement.
    *
@@ -61,21 +62,25 @@ namespace mariadb
       connection(_connection),
       protocol(_connection->getProtocol()),
       lock(_connection->lock),
-      resultSetScrollType(_resultSetScrollType),
       resultSetConcurrency(_resultSetConcurrency),
       options(protocol->getOptions()),
       canUseServerTimeout(_connection->canUseServerTimeout()),
       exceptionFactory(factory),
       fetchSize(options->defaultFetchSize),
-      batchRes(0),
-      largeBatchRes(0)
+      batchRes(0LL),
+      largeBatchRes(0LL)
   {
+    // This would check and reset fetchSize, if not FORWARD_ONLY
+    setResultSetType(_resultSetScrollType);
   }
 
 
   Statement* MariaDbStatement::setResultSetType(int32_t rsType)
   {
     resultSetScrollType= rsType;
+    if (resultSetScrollType != ResultSet::TYPE_FORWARD_ONLY) {
+      setFetchSize(0);
+    }
     return this;
   }
 
@@ -98,8 +103,10 @@ namespace mariadb
 
   MariaDbStatement::~MariaDbStatement()
   {
+    // We need to close associated resultset(the last one, previous should be closed once next result is requested)
     if (results) {
       results->loadFully(true, protocol.get());
+      results->close();
     }
   }
 
@@ -144,6 +151,10 @@ namespace mariadb
   void MariaDbStatement::executeQueryPrologue(bool isBatch) {
     setExecutingFlag();
     if (closed) {
+      logger->trace("Query Prolog:", std::hex, this, "Closed: ", true, "Connection:", connection, "Protocol:", protocol.get(), "Closed: ", protocol ? protocol->isClosed(): true);
+      if (connection) {
+        logger->trace("QP: Connection closed: ", connection->isClosed());
+      }
       exceptionFactory->raiseStatementError(connection, this)->create("execute() is called on closed statement").Throw();
     }
     protocol->prolog(maxRows, protocol->getProxy(), connection, this);
@@ -169,7 +180,7 @@ namespace mariadb
 
         }
       }
-      timerTaskFuture= NULL;
+      timerTaskFuture= nullptr;
     }
 #endif
   }
@@ -189,8 +200,8 @@ namespace mariadb
       catch (SQLException& ) {
       }
     }
-
-    if (sqle.getErrorCode()==1148 &&!options->allowLocalInfile){
+    auto errorCode= sqle.getErrorCode();
+    if ((errorCode == 1148 || errorCode == 4166) && !options->allowLocalInfile) {
       return exceptionFactory->raiseStatementError(connection, this)->create(
           "Usage of LOCAL INFILE is disabled. "
           "To use it enable it via the connection property allowLocalInfile=true",
@@ -199,7 +210,7 @@ namespace mariadb
           &sqle);
     }
 
-    if (isTimedout){
+    if (isTimedout) {
       return exceptionFactory->raiseStatementError(connection, this)->create("Query timed out", "70100", 1317, &sqle);
     }
     MariaDBExceptionThrower sqlException= exceptionFactory->raiseStatementError(connection, this)->create(sqle);
@@ -216,24 +227,22 @@ namespace mariadb
     setExecutingFlag(false);
   }
 
-  void MariaDbStatement::executeBatchEpilogue(){
+  void MariaDbStatement::executeBatchEpilogue() {
     setExecutingFlag(false);
     stopTimeoutTask();
     isTimedout= false;
     clearBatch();
   }
 
-  MariaDBExceptionThrower MariaDbStatement::handleFailoverAndTimeout(SQLException& sqle)
-  {
-    if (sqle.getSQLState().empty() != true && sqle.getSQLState().startsWith("08")){
+  MariaDBExceptionThrower MariaDbStatement::handleFailoverAndTimeout(SQLException& sqle) {
+    if (sqle.getSQLState().empty() != true && sqle.getSQLState().startsWith("08")) {
       try {
         close();
-      }catch (SQLException&){
-
+      }
+      catch (SQLException&) {
       }
     }
-
-    if (isTimedout){
+    if (isTimedout) {
       return exceptionFactory->raiseStatementError(connection, this)->create("Query timed out", "70100", 1317, &sqle);
     }
     MariaDBExceptionThrower exThrower;
@@ -281,10 +290,8 @@ namespace mariadb
     std::unique_lock<std::mutex> localScopeLock(*lock);
 
     try {
-      std::vector<Shared::ParameterHolder> dummy;
       executeQueryPrologue(false);
-      results.reset(
-        new Results(
+      results.reset(new Results(
             this,
             fetchSize,
             false,
@@ -295,9 +302,9 @@ namespace mariadb
             autoGeneratedKeys,
             protocol->getAutoIncrementIncrement(),
             sql,
-            dummy));
-
-      protocol->executeQuery(protocol->isMasterConnection(), results, getTimeoutSql(Utils::nativeSql(sql, protocol.get())));
+            nullptr));
+      SQLString nativeSql, timeoutSql;
+      protocol->executeQuery(protocol->isMasterConnection(), results.get(), getTimeoutSql(Utils::nativeSql(sql, nativeSql, protocol.get()), timeoutSql));
 
       results->commandEnd();
       executeEpilogue();
@@ -415,10 +422,18 @@ namespace mariadb
   }
 
 
-  SQLString MariaDbStatement::getTimeoutSql(const SQLString& sql)
+  const SQLString& MariaDbStatement::getTimeoutSql(const SQLString& sql, SQLString& buffer)
   {
     if (queryTimeout > 0 && canUseServerTimeout) {
-      return "SET STATEMENT max_statement_time="+ std::to_string(queryTimeout) +" FOR "+ sql;
+      std::string &buffStr= StringImp::get(buffer);
+      const std::string &sqlStr= StringImp::get(buffer);
+      constexpr std::size_t setStmtLen= sizeof("SET STATEMENT max_statement_time=") - 1,
+        forLen= 5/* FOR */,
+        additionalLen= setStmtLen + 10/*int32_t max str len*/ + forLen;
+      buffStr.reserve(sqlStr.size() + additionalLen);
+      buffStr.append("SET STATEMENT max_statement_time=", setStmtLen).append(std::to_string(queryTimeout)).
+        append(" FOR ", forLen).append(sqlStr);
+      return buffer;
     }
     return sql;
   }
@@ -435,7 +450,6 @@ namespace mariadb
   {
     std::lock_guard<std::mutex> localScopeLock(*lock);
     try {
-      std::vector<Shared::ParameterHolder> dummy;
       executeQueryPrologue(false);
       results.reset(new Results(
             this,
@@ -448,12 +462,12 @@ namespace mariadb
             Statement::NO_GENERATED_KEYS,
             protocol->getAutoIncrementIncrement(),
             sql,
-            dummy));
-
+            nullptr));
+      SQLString nativeSql, timeoutSql;
       protocol->executeQuery(
           protocol->isMasterConnection(),
-          results,
-          getTimeoutSql(Utils::nativeSql(sql, protocol.get())),
+          results.get(),
+          getTimeoutSql(Utils::nativeSql(sql, nativeSql, protocol.get()), timeoutSql),
           &charset);
 
       results->commandEnd();
@@ -602,7 +616,7 @@ namespace mariadb
    * @throws SQLException if the query could not be sent to server.
    */
   int32_t MariaDbStatement::executeUpdate(const SQLString& sql) {
-    if (executeInternal(sql,fetchSize,Statement::NO_GENERATED_KEYS)){
+    if (executeInternal(sql, fetchSize, Statement::NO_GENERATED_KEYS)) {
       throw SQLException("executeUpdate should not be used for queries returning a resultset");
     }
     return getUpdateCount();
@@ -723,29 +737,28 @@ namespace mariadb
    */
   void MariaDbStatement::close()
   {
-    std::lock_guard<std::mutex> localScopeLock(*lock);
-
     try {
-      closed = true;
+      // skipMoreResults acquires mutex, thux no need to do that here
+      closed= true;
       if (results) {
         if (results->getFetchSize() != 0) {
           skipMoreResults();
         }
-
         results->close();
       }
 
-      if (protocol->isClosed()
-        || !connection->pooledConnection
-        || connection->pooledConnection->noStmtEventListeners()) {
-        //protocol.reset();
-        return;
+      if (connection && !((protocol && protocol->isClosed())
+        || !connection->poolConnection
+        || connection->poolConnection->noStmtEventListeners())) {
+        // Commenting this out so far, as fireStatementClosed does not do anything, and it's not even clear
+        // if that lock was needed here in first turn
+        //std::lock_guard<std::mutex> localScopeLock(*lock);
+        connection->poolConnection->fireStatementClosed(this);
       }
-      connection->pooledConnection->fireStatementClosed(this);
     }
     catch (...) {
     }
-    //protocol.reset();
+    protocol= nullptr;
     connection= nullptr;
   }
 
@@ -869,6 +882,7 @@ namespace mariadb
     this->queryTimeout= seconds;
   }
 
+#ifdef WE_DO_OWN_PROTOCOL_IMPEMENTATION
   /**
    * Sets the inputStream that will be used for the next execute that uses "LOAD DATA LOCAL INFILE".
    * The name specified as local file/URL will be ignored.
@@ -881,6 +895,7 @@ namespace mariadb
     checkClose();
     protocol->setLocalInfileInputStream(*inputStream);
   }
+#endif
 
   /**
    * Cancels this <code>Statement</code> object if both the DBMS and driver support aborting an SQL
@@ -949,7 +964,7 @@ namespace mariadb
     if (!warningsCleared){
       return this->connection->getWarnings();
     }
-    return NULL;
+    return nullptr;
   }
 
   /**
@@ -1035,7 +1050,7 @@ namespace mariadb
    * @since 1.6
    */
   bool MariaDbStatement::isClosed(){
-    return closed;
+    return closed || protocol->isExplicitClosed();
   }
 
 
@@ -1205,12 +1220,14 @@ namespace mariadb
    * @see #getFetchSize
    */
   void MariaDbStatement::setFetchSize(int32_t rows) {
-    if (rows != 0) {
-      throw SQLFeatureNotImplementedException("setFetchSize(ResultSet streaming) support is implemented beginning from version 1.1");
-    }
-    if (rows < 0 && rows != INT32_MIN){
+    if (rows < 0 && rows != INT32_MIN) {
       exceptionFactory->raiseStatementError(connection, this)->create("invalid fetch size").Throw();
-    }else if (rows == INT32_MIN)
+    }
+    else if (rows != 0 && getResultSetType() != ResultSet::TYPE_FORWARD_ONLY) {
+      exceptionFactory->raiseStatementError(connection, this)->create("ResultSet Streaming is not supported for for ResultSet "
+        "types other than ResultSet::TYPE_FORWARD_ONLY").Throw();
+    }
+    else if (rows == INT32_MIN)
     {
       this->fetchSize= 1;
       return;
@@ -1358,7 +1375,6 @@ namespace mariadb
    */
   void MariaDbStatement::internalBatchExecution(std::size_t size)
   {
-    std::vector<Shared::ParameterHolder> dummy;
     executeQueryPrologue(true);
     results.reset(new Results(
           this,
@@ -1370,9 +1386,9 @@ namespace mariadb
           resultSetConcurrency,
           Statement::RETURN_GENERATED_KEYS,
           protocol->getAutoIncrementIncrement(),
-          NULL,
-          dummy));
-    protocol->executeBatchStmt(protocol->isMasterConnection(),results,batchQueries);
+          nullptr,
+          nullptr));
+    protocol->executeBatchStmt(protocol->isMasterConnection(), results.get(), batchQueries);
     results->commandEnd();
   }
 
@@ -1460,7 +1476,7 @@ namespace mariadb
   void MariaDbStatement::markClosed()
   {
     closed= true;
-    connection= NULL;
+    connection= nullptr;
   }
 }
 }
