@@ -109,9 +109,12 @@ namespace sql
         "Metadata ResultSetMetaData.getTableName() returns the physical table name. \"useOldAliasMetadataBehavior\""
         " permits activating the legacy code that sends the table alias if set.",
         false,
-        false}},
+        false}
+      },
       {
-        "allowLocalInfile", {"allowLocalInfile", "1.0.2", "Permits loading data from local file(on the client)", false, false}},
+        "allowLocalInfile", { "allowLocalInfile",
+        "1.1.1", "Permits loading data from local file(on the client)", false, false}
+      },
       {
         "sessionVariables", {"sessionVariables",
         "0.9.1",
@@ -281,13 +284,13 @@ namespace sql
         int32_t(0)}},
       {
         "cachePrepStmts", {"cachePrepStmts",
-        "0.9.1",
-        "enable/disable prepare Statement cache, default true.",
+        "1.1.3",
+        "enable/disable prepare Statement cache, default false.",
         false,
         false}},
       {
         "prepStmtCacheSize", {"prepStmtCacheSize",
-        "0.9.1",
+        "1.1.3",
         "This sets the number of prepared statements that the "
         "driver will cache per connection if \"cachePrepStmts\" is enabled.",
         false,
@@ -295,9 +298,9 @@ namespace sql
         int32_t(0)}},
       {
         "prepStmtCacheSqlLimit", {"prepStmtCacheSqlLimit",
-        "0.9.1",
-        "This is the maximum length of a prepared SQL"
-        " statement that the driver will cache  if \"cachePrepStmts\" is enabled.",
+        "1.1.3",
+        "This is the maximum length of a (SQL query length + schema name length + 1) "
+        "the statement that the driver will cache  if \"cachePrepStmts\" is enabled.",
         false,
         (int32_t)2048,
         int32_t(0)}},
@@ -400,8 +403,7 @@ namespace sql
         "tlsCRLPath", {"tlsCRLPath",
         "0.9.2",
         "A path to a directory that contains one or more PEM files that should each contain one revoked X509 certificate.\n"
-        "The directory specified by this option needs to be run through the openssl rehash command.\n"
-        " ",
+        "The directory specified by this option needs to be run through the openssl rehash command.",
         false,
         ""}
       },
@@ -459,8 +461,9 @@ namespace sql
         "(example: connectionAttributes=key1:value1,key2,value2).\n"
         "This information can be retrieved on server within tables performance_schema.session_connect_attrs "
         "and performance_schema.session_account_connect_attrs.\n"
-        "This allowa an identification of client/application on server",
-        false}},
+        "This allows the identification of client/application on the server",
+        false}
+      },
       {
         "useBatchMultiSend", {"useBatchMultiSend",
         "0.9.1",
@@ -484,13 +487,19 @@ namespace sql
         (int32_t)100,
         int32_t(1)}},
       {
-        "log", {"log",
-        "0.9.1",
+        "log", {"log", (uint32_t)5,
+        "1.1.3",
         "Enable log information. \n"
-        "require Slf4j version > 1.4 dependency.\n"
-        "Log level correspond to Slf4j logging implementation",
-        false,
-        false}},
+        "The value defines logging level: 1 - error, 2 - warning, 3 - info, 4 - debug, 5 - trace" }
+      },
+      {
+        "logname", {"logname",
+        "1.1.3",
+        "File name for the log. The default name is mariadbccpp.log,"
+        "and default location is home dir if found, or directory for temporary files\n"
+        "(%TMP% or /tmp)",
+        false}
+      },
       {"profileSql", {"profileSql", "0.9.1", "log query execution time.", false, false}},
       {"maxQuerySizeToLog", {"maxQuerySizeToLog", "0.9.1", "Max query log size.", false, 1024, 0}},
       {
@@ -549,7 +558,7 @@ namespace sql
         "possible(batch without Statement::RETURN_GENERATED_KEYS and streams). Can be significanlty faster."
         "(works only with server MariaDB >= 10.2.7)",
         false,
-        false}},
+        true}},
       {
         "autocommit", {"autocommit",
         "0.9.1",
@@ -705,6 +714,13 @@ namespace sql
         false,
         ""}
       },
+      {
+        "initSql", {"initSql",
+        "1.1.7",
+        "SQL command(s) to run at connection time",
+        false,
+        ""}
+      },
       {
         "restrictedAuth", {"restrictedAuth",
         "1.0.6",
@@ -760,14 +776,14 @@ namespace sql
     std::map<std::string, DefaultOptions*> DefaultOptions::OPTIONS_MAP;
     static bool aliasesAdded= addAliases(DefaultOptions::OPTIONS_MAP);
 //-------------------------------------------------------------------------------------------------------------------------------------
-    DefaultOptions::DefaultOptions(const char * optionName, const char * /*implementationVersion*/, const char* description, bool required)
-      : optionName(optionName)
-      , description(description), required(required)
-      , minValue()
-      , maxValue()
-      , defaultValue("")
-    {
-    }
+    //DefaultOptions::DefaultOptions(const char * optionName, const char * /*implementationVersion*/, const char* description, bool required, const char* defaultVal)
+    //  : optionName(optionName)
+    //  , description(description), required(required)
+    //  , minValue()
+    //  , maxValue()
+    //  , defaultValue(defaultVal)
+    //{
+    //}
 
     DefaultOptions::DefaultOptions(const char * optionName, const char * /*implementationVersion*/, const char * description,
         bool required, const char * defaultValue) :
@@ -813,6 +829,17 @@ namespace sql
     {
     }
 
+    DefaultOptions::DefaultOptions(const char* _optionName, uint32_t _maxValue, const char* /*implementationVersion*/,
+      const char* _description, uint32_t _defaultValue, bool _required) :
+      optionName(_optionName),
+      description(_description),
+      required(_required),
+      minValue(0U),
+      maxValue(_maxValue),
+      defaultValue(_defaultValue)
+    {
+    }
+
 #ifdef WE_NEED_INT_ARRAY_DEFAULT_VALUE
     DefaultOptions::DefaultOptions(const SQLString& optionName, int32_t* defaultValue, int32_t minValue,
       const SQLString& /*implementationVersion*/, SQLString& description, bool required) :
@@ -828,7 +855,7 @@ namespace sql
 
     Shared::Options DefaultOptions::defaultValues(HaMode haMode)
     {
-      Properties properties;
+      PropertiesImp::ImpType properties;
       return parse(haMode, emptyStr, properties);
     }
     /**
@@ -840,10 +867,10 @@ namespace sql
      */
     Shared::Options DefaultOptions::defaultValues(HaMode haMode, bool pool)
     {
-      Properties properties;
+      PropertiesImp::ImpType  properties;
       properties.insert({ "pool", pool ? "true" : "false" });
       Shared::Options options= parse(haMode, emptyStr, properties);
-      postOptionProcess(options, nullptr);
+      postOptionProcess(options.get(), nullptr);
       return options;
     }
 
@@ -856,15 +883,15 @@ namespace sql
      */
     void DefaultOptions::parse(const enum HaMode haMode, const SQLString& urlParameters, Shared::Options options)
     {
-      Properties prop;
+      PropertiesImp::ImpType prop;
       parse(haMode, urlParameters, prop, options);
-      postOptionProcess(options, nullptr);
+      postOptionProcess(options.get(), nullptr);
     }
 
 
-    Shared::Options DefaultOptions::parse( const enum HaMode haMode,const SQLString& urlParameters, Properties& properties) {
+    Shared::Options DefaultOptions::parse( const enum HaMode haMode,const SQLString& urlParameters, PropertiesImp::ImpType& properties) {
       Shared::Options options= parse(haMode, urlParameters, properties, nullptr);
-      postOptionProcess(options, nullptr);
+      postOptionProcess(options.get(), nullptr);
       return options;
     }
 
@@ -877,10 +904,10 @@ namespace sql
      * @param options initial options
      * @return options
      */
-    Shared::Options DefaultOptions::parse(enum HaMode haMode, const SQLString& urlParameters, Properties& properties,
+    Shared::Options DefaultOptions::parse(enum HaMode haMode, const SQLString& urlParameters, PropertiesImp::ImpType& properties,
       Shared::Options options)
     {
-      if (/*urlParameters != nullptr &&*/ !urlParameters.empty())
+      if (!urlParameters.empty())
       {
         Tokens parameters= split(urlParameters, "&");
 
@@ -902,7 +929,7 @@ namespace sql
     }
 
 
-    Shared::Options DefaultOptions::parse(enum HaMode haMode, const Properties& properties, Shared::Options paramOptions)
+    Shared::Options DefaultOptions::parse(enum HaMode haMode, const PropertiesImp::ImpType& properties, Shared::Options paramOptions)
     {
       Options &options= *paramOptions;
 
@@ -920,18 +947,19 @@ namespace sql
             DefaultOptions *o= cit->second;
             const ClassField<Options> field= Options::getField(o->optionName);
 
-            if (o->objType() == Value::VSTRING ){
-              field.set(options, propertyValue);
-            }
-            else if (o->objType() == Value::VBOOL)
+            switch (o->objType())
             {
-              if (propertyValue.toLowerCase().length() == 0
+            case Value::VSTRING:
+              field.set(options, propertyValue);
+              break;
+            case Value::VBOOL:
+              if (!StringImp::isNull(propertyValue) && (propertyValue.toLowerCase().length() == 0
                 || propertyValue.compare("1") == 0
-                || propertyValue.compare("true") == 0)
+                || propertyValue.compare("true") == 0))
               {
                 field.set(options, true);
               }
-              else if (propertyValue.compare("0") == 0
+              else if (StringImp::isNull(propertyValue) || propertyValue.compare("0") == 0
                 || propertyValue.compare("false") == 0)
               {
                 field.set(options, false);
@@ -939,15 +967,14 @@ namespace sql
               else
               {
                 throw IllegalArgumentException(
-                    "Optional parameter "
-                    +o->optionName
-                    +" must be bool (true/false or 0/1) was \""
-                    +propertyValue
-                    +"\"");
+                  "Optional parameter "
+                  + o->optionName
+                  + " must be bool (true/false or 0/1) was \""
+                  + propertyValue
+                  + "\"");
               }
-            }
-            else if (o->objType() == Value::VINT32)
-            {
+              break;
+            case Value::VINT32:
               try {
                 int32_t value= std::stoi(StringImp::get(propertyValue));
 
@@ -957,19 +984,19 @@ namespace sql
                 int32_t maxValue= o->maxValue;
 
                 if (value < minValue
-                    || (maxValue != INT32_MAX && value > maxValue))
+                  || (maxValue != INT32_MAX && value > maxValue))
                 {
                   throw IllegalArgumentException(
-                      "Optional parameter "
-                      +o->optionName
-                      +" must be greater or equal to "
-                      + o->minValue.toString()
-                      + (maxValue != INT32_MAX
-                        ?" and smaller than " + o->maxValue.toString()
-                        :" ")
-                      +", was \""
-                      +propertyValue
-                      +"\"");
+                    "Optional parameter "
+                    + o->optionName
+                    + " must be greater or equal to "
+                    + o->minValue.toString()
+                    + (maxValue != INT32_MAX
+                      ? " and smaller than " + o->maxValue.toString()
+                      : " ")
+                    + ", was \""
+                    + propertyValue
+                    + "\"");
                 }
 
                 field.set(options, value);
@@ -977,15 +1004,51 @@ namespace sql
               catch (std::invalid_argument&)
               {
                 throw IllegalArgumentException(
+                  "Optional parameter "
+                  + o->optionName
+                  + " must be Integer, was \""
+                  + propertyValue
+                  + "\"");
+              }
+              break;
+            case Value::VUINT32:
+              try {
+                uint64_t value= std::stoul(StringImp::get(propertyValue));
+
+                assert(!o->minValue.empty());
+                assert(!o->maxValue.empty());
+                uint32_t minValue= o->minValue;
+                uint32_t maxValue= o->maxValue;
+
+                if (value < minValue
+                  || (maxValue != 0 && value > maxValue) || value > UINT32_MAX)
+                {
+                  throw IllegalArgumentException(
                     "Optional parameter "
                     + o->optionName
-                    + " must be Integer, was \""
+                    + " must be in uint32_t range, and greater or equal to "
+                    + o->minValue.toString()
+                    + (maxValue != UINT32_MAX
+                      ? " and smaller than " + o->maxValue.toString()
+                      : " ")
+                    + ", was \""
                     + propertyValue
                     + "\"");
+                }
+
+                field.set(options, static_cast<uint32_t>(value));
               }
-            }
-            else if (o->objType() == Value::VINT64)
-            {
+              catch (std::invalid_argument&)
+              {
+                throw IllegalArgumentException(
+                  "Optional parameter "
+                  + o->optionName
+                  + " must be Integer, was \""
+                  + propertyValue
+                  + "\"");
+              }
+              break;
+            case Value::VINT64:
               try {
                 int64_t value= std::stoll(StringImp::get(propertyValue));
 
@@ -996,31 +1059,33 @@ namespace sql
                 int64_t maxValue= o->maxValue;
 
                 if (value < minValue
-                    || (maxValue != INT64_MAX && value > maxValue))
+                  || (maxValue != INT64_MAX && value > maxValue))
                 {
                   throw IllegalArgumentException(
-                      "Optional parameter "
-                      + o->optionName
-                      + " must be greater or equal to "
-                      + o->minValue.toString()
-                      + (maxValue !=INT64_MAX
-                        ? " and smaller than " + o->maxValue.toString()
-                        : SQLString(" "))
-                      + ", was \""
-                      + propertyValue
-                      + "\"");
+                    "Optional parameter "
+                    + o->optionName
+                    + " must be greater or equal to "
+                    + o->minValue.toString()
+                    + (maxValue != INT64_MAX
+                      ? " and smaller than " + o->maxValue.toString()
+                      : SQLString(" "))
+                    + ", was \""
+                    + propertyValue
+                    + "\"");
                 }
-                field.set(options,value);
+                field.set(options, value);
               }
               catch (std::invalid_argument&)
               {
                 throw IllegalArgumentException(
-                    "Optional parameter "
-                    +o->optionName
-                    +" must be Long, was \""
-                    +propertyValue
-                    +"\"");
+                  "Optional parameter "
+                  + o->optionName
+                  + " must be Long, was \""
+                  + propertyValue
+                  + "\"");
               }
+            default:
+              break;
             }
           }
           else
@@ -1047,7 +1112,7 @@ namespace sql
      * @param options options
      * @param credentialPlugin credential plugin
      */
-    void DefaultOptions::postOptionProcess(const Shared::Options options, CredentialPlugin* credentialPlugin)
+    void DefaultOptions::postOptionProcess(Options* options, CredentialPlugin* credentialPlugin)
     {
       if (options->rewriteBatchedStatements){
         options->useServerPrepStmts= false;
@@ -1062,16 +1127,14 @@ namespace sql
           options->minPoolSize == 0
           ? options->maxPoolSize
           : std::min(options->minPoolSize, options->maxPoolSize);
-
-        throw SQLFeatureNotImplementedException("This connector version does not have pool support");
       }
 
-      if (options->cacheCallableStmts || options->cachePrepStmts) {
-        throw SQLFeatureNotImplementedException("Callable/Prepared statement caches are not supported yet");
+      if (options->cacheCallableStmts) {// || options->cachePrepStmts) {
+        throw SQLFeatureNotImplementedException("Callable statement caches are not supported yet");
       }
 
-      if (options->defaultFetchSize != 0){
-        throw SQLFeatureNotImplementedException("ResultSet streaming is not supported in this version");
+      if (options->defaultFetchSize < 0) {
+        options->defaultFetchSize= 0;
       }
 
       if (credentialPlugin && credentialPlugin->mustUseSsl())
@@ -1086,6 +1149,10 @@ namespace sql
       if (options->useCharacterEncoding.compare("utf8") == 0) {
         options->useCharacterEncoding = "utf8mb4";
       }
+
+      if (!options->logname.empty() && options->log == 0) {
+        options->log= SimpleLogger::ERROR;
+      }
     }
 
     /**
