@@ -21,6 +21,8 @@
 #include <random>
 #include <chrono>
 
+#include "util/ServerPrepareStatementCache.h"
+
 #include "ConnectProtocol.h"
 
 #include "logger/LoggerFactory.h"
@@ -46,7 +48,7 @@ namespace capi
     "@@time_zone,"
     "@@auto_increment_increment");
   const SQLString ConnectProtocol::IS_MASTER_QUERY("select @@innodb_read_only");
-  Shared::Logger ConnectProtocol::logger(LoggerFactory::getLogger(typeid(ConnectProtocol)));
+  Logger* ConnectProtocol::logger= LoggerFactory::getLogger(typeid(ConnectProtocol));
   static const SQLString MARIADB_RPL_HACK_PREFIX("5.5.5-");
 
   /**
@@ -56,10 +58,10 @@ namespace capi
    * @param globalInfo server global variables information
    * @param lock the lock for thread synchronisation
    */
-  ConnectProtocol::ConnectProtocol(std::shared_ptr<UrlParser>& _urlParser, GlobalStateInfo* _globalInfo, Shared::mutex& lock)
+  ConnectProtocol::ConnectProtocol(std::shared_ptr<UrlParser>& _urlParser, GlobalStateInfo* _globalInfo)
     :
       connection(nullptr, &mysql_close)
-    , lock(lock)
+    , lock()
     , urlParser(_urlParser)
     , options(_urlParser->getOptions())
     , username(_urlParser->getUsername())
@@ -70,8 +72,11 @@ namespace capi
     , currentHost(localhost, 3306)
   {
     urlParser->auroraPipelineQuirks();
-    if (options->cachePrepStmts && options->useServerPrepStmts){
-      //ServerPrepareStatementCache::newInstance(options->prepStmtCacheSize, this);
+    if (options->cachePrepStmts && options->useServerPrepStmts) {
+      serverPrepareStatementCache.reset(new ServerPrepareStatementCache(options->prepStmtCacheSize, static_cast<std::size_t>(options->prepStmtCacheSqlLimit)));
+    }
+    else {
+      serverPrepareStatementCache.reset(new NoCache());
     }
   }
 
@@ -173,7 +178,7 @@ namespace capi
     }
 
     if (options->useCompression){
-      if ((serverCapabilities &MariaDbServerCapabilities::COMPRESS)==0){
+      if ((serverCapabilities & MariaDbServerCapabilities::COMPRESS)==0){
 
         options->useCompression= false;
       }else {
@@ -209,7 +214,7 @@ namespace capi
     if (!options->enabledTlsProtocolSuites.empty()) {
       Tokens protocols= split(options->enabledTlsProtocolSuites, "[,;\\s]+");
       for (const auto& protocol : *protocols){
-        if (StringImp::get(possibleProtocols).find(protocol) == std::string::npos){
+        if (possibleProtocols.find(protocol) == std::string::npos){
           throw SQLException(
               "Unsupported TLS protocol '"
               +protocol
@@ -256,7 +261,7 @@ namespace capi
     }
     catch (std::runtime_error& ) {
     }
-    std::unique_lock<std::mutex> localScopeLock(*lock);
+    std::unique_lock<std::mutex> localScopeLock(lock);
     // We still can statements waiting to be closed
     forceReleaseWaitingPrepareStatement();
     closeSocket();
@@ -266,12 +271,13 @@ namespace capi
   /** Force closes socket and stream readers/writers. */
   void ConnectProtocol::abort()
   {
+    GET_LOGGER()->trace("Protocol::abort:", std::hex, this);
     this->explicitClosed= true;
     bool lockStatus= false;
 
-    if (lock){
-      lockStatus= lock->try_lock();
-    }
+    //if (lock) {
+    lockStatus= lock.try_lock();
+    //}
     this->connected= false;
 
     abortActiveStream();
@@ -292,15 +298,14 @@ namespace capi
     cleanMemory();
 
     if (lockStatus){
-      lock->unlock();
+      lock.unlock();
     }
   }
 
   void ConnectProtocol::forceAbort()
   {
     try {
-      Shared::mutex forCopied(new std::mutex());
-      std::unique_ptr<MasterProtocol> copiedProtocol(new MasterProtocol(urlParser, new GlobalStateInfo(), forCopied));
+      std::unique_ptr<MasterProtocol> copiedProtocol(new MasterProtocol(urlParser, new GlobalStateInfo()));
       copiedProtocol->setHostAddress(getHostAddress());
       copiedProtocol->connect();
 
@@ -338,9 +343,7 @@ namespace capi
 
   void ConnectProtocol::cleanMemory()
   {
-    if (options->cachePrepStmts && options->useServerPrepStmts && serverPrepareStatementCache){
-      //serverPrepareStatementCache.clear();
-    }
+    serverPrepareStatementCache->clear();
     if (options->enablePacketDebug){
       //traceCache->clearMemory();
     }
@@ -452,9 +455,17 @@ namespace capi
     mysql_optionsv(connection.get(), MYSQL_REPORT_DATA_TRUNCATION, &uintOptionSelected);
     mysql_optionsv(connection.get(), MYSQL_OPT_LOCAL_INFILE, (options->allowLocalInfile ? &uintOptionSelected : &uintOptionNotSelected));
 
+    if (!options.get()->initSql.empty()) {
+      static const SQLString initSqlDelim(";");
+      auto Query= split(options.get()->initSql, initSqlDelim);
+      for (auto& sql : *Query) {
+        mysql_optionsv(connection.get(), MYSQL_INIT_COMMAND, sql.c_str());
+      }
+    }
     if (!options.get()->restrictedAuth.empty()) {
       mysql_optionsv(connection.get(), MARIADB_OPT_RESTRICTED_AUTH, options.get()->restrictedAuth.c_str());
     }
+
     if (mysql_real_connect(connection.get(), NULL, NULL, NULL, NULL, 0, NULL, CLIENT_MULTI_STATEMENTS) == nullptr)
     {
       throw SQLException(mysql_error(connection.get()), mysql_sqlstate(connection.get()), mysql_errno(connection.get()));
@@ -472,7 +483,7 @@ namespace capi
       serverVersion= serverVersion.substr(MARIADB_RPL_HACK_PREFIX.length());
     }
     else {
-      serverMariaDb= StringImp::get(serverVersion).find("MariaDB") != std::string::npos;
+      serverMariaDb= serverVersion.find("MariaDB") != std::string::npos;
     }
     unsigned long baseCaps, extCaps;
     mariadb_get_infov(connection.get(), MARIADB_CONNECTION_EXTENDED_SERVER_CAPABILITIES, (void*)&extCaps);
@@ -553,7 +564,6 @@ namespace capi
       mysql_optionsv(connection.get(), MARIADB_OPT_TLS_PEER_FP, options->tlsPeerFP.c_str());
     }
 
-
     // This is not quite a TLS option, but still putting it here
     if (!options->serverRsaPublicKeyFile.empty()) {
       mysql_optionsv(connection.get(), MYSQL_SERVER_PUBLIC_KEY, (void*)options->serverRsaPublicKeyFile.c_str());
@@ -713,7 +723,6 @@ namespace capi
 
   void ConnectProtocol::sendSessionInfos()
   {
-
     SQLString sessionOption("autocommit=");
     sessionOption.append(options->autocommit ? "1" : "0");
 
@@ -743,20 +752,21 @@ namespace capi
 
   void ConnectProtocol::readRequestSessionVariables(std::map<SQLString, SQLString>& serverData)
   {
+    // This is very strange piece of code
     Unique::Results results(new Results());
     getResult(results.get());
 
     results->commandEnd();
     ResultSet* resultSet= results->getResultSet();
-    if (resultSet){
+    if (resultSet) {
       resultSet->next();
 
       serverData.emplace("max_allowed_packet",resultSet->getString(1));
       serverData.emplace("system_time_zone",resultSet->getString(2));
       serverData.emplace("time_zone",resultSet->getString(3));
       serverData.emplace("auto_increment_increment", resultSet->getString(4));
-
-    }else {
+    }
+    else {
       throw SQLException(mysql_get_socket(connection.get()) == MARIADB_INVALID_SOCKET ?
         "Error reading SessionVariables results. Socket is NOT connected" :
         "Error reading SessionVariables results. Socket IS connected");
@@ -777,14 +787,13 @@ namespace capi
 
   void ConnectProtocol::readPipelineAdditionalData(std::map<SQLString, SQLString>& serverData)
   {
-
     MariaDBExceptionThrower resultingException;
 
     try {
       Unique::Results res(new Results());
       getResult(res.get());
-    }catch (SQLException& sqlException){
-
+    }
+    catch (SQLException& sqlException) {
       resultingException.take(sqlException);
     }
 
@@ -792,8 +801,9 @@ namespace capi
 
     try {
       readRequestSessionVariables(serverData);
-    }catch (SQLException& sqlException){
-      if (!resultingException){
+    }
+    catch (SQLException& sqlException) {
+      if (!resultingException) {
         resultingException.assign(exceptionFactory->create("could not load system variables", "08000", &sqlException));
         canTrySessionWithShow= true;
       }
@@ -801,16 +811,16 @@ namespace capi
 
     try {
       readPipelineCheckMaster();
-    }catch (SQLException& sqlException){
+    }
+    catch (SQLException& sqlException) {
       canTrySessionWithShow= false;
-      if (!resultingException){
+      if (!resultingException) {
         exceptionFactory->create(
             "could not identified if server is master", "08000", &sqlException).Throw();
       }
     }
 
-    if (canTrySessionWithShow){
-
+    if (canTrySessionWithShow) {
       requestSessionDataWithShow(serverData);
       connected= true;
       return;
@@ -826,10 +836,10 @@ namespace capi
   void ConnectProtocol::requestSessionDataWithShow(std::map<SQLString, SQLString>& serverData)
   {
     try {
-      Shared::Results results(new Results());
+      Unique::Results results(new Results());
       executeQuery(
           true,
-          results,
+          results.get(),
           "SHOW VARIABLES WHERE Variable_name in ("
           "'max_allowed_packet',"
           "'system_time_zone',"
@@ -872,7 +882,7 @@ namespace capi
     sendPipelineCheckMaster();
     readPipelineCheckMaster();
 
-    if (options->createDatabaseIfNotExist && !database.empty()){
+    if (options->createDatabaseIfNotExist && !database.empty()) {
 
       SQLString quotedDb(MariaDbConnection::quoteIdentifier(this->database));
       sendCreateDatabaseIfNotExist(quotedDb);
@@ -898,7 +908,6 @@ namespace capi
 
   void ConnectProtocol::loadCalendar(const SQLString& /*srvTimeZone*/, const SQLString& /*srvSystemTimeZone*/)
   {
-
     timeZone= nullptr;// Calendar.getInstance().getTimeZone();
 
 #ifdef WE_HAVE_NEED_TIMEZONE
@@ -1287,7 +1296,7 @@ namespace capi
    */
   bool ConnectProtocol::hasWarnings()
   {
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
     return hasWarningsFlag;
   }
 
@@ -1298,7 +1307,7 @@ namespace capi
    */
   bool ConnectProtocol::isConnected()
   {
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
     return connected;
   }
 
@@ -1352,9 +1361,9 @@ namespace capi
     }
   }
 
-  Shared::mutex& ConnectProtocol::getLock()
+  std::mutex *const ConnectProtocol::getLock()
   {
-    return lock;
+    return &lock;
   }
 
   bool ConnectProtocol::hasMoreResults()
@@ -1362,9 +1371,9 @@ namespace capi
     return (serverStatus & ServerStatus::MORE_RESULTS_EXISTS) != 0;
   }
 
-  ServerPrepareStatementCache* ConnectProtocol::prepareStatementCache()
+  Cache* ConnectProtocol::prepareStatementCache()
   {
-    return serverPrepareStatementCache;
+    return serverPrepareStatementCache.get();
   }
 
   /**
@@ -1382,6 +1391,7 @@ namespace capi
     }
   }
 
+
   void ConnectProtocol::changeSocketSoTimeout(int32_t millis)
   {
     this->socketTimeout = millis;
@@ -1391,6 +1401,7 @@ namespace capi
     mysql_optionsv(connection.get(), MYSQL_OPT_READ_TIMEOUT, (void*)&millis);
   }
 
+
   bool ConnectProtocol::isServerMariaDb()
   {
     return serverMariaDb;
@@ -1492,7 +1503,7 @@ namespace capi
   }
   void ConnectProtocol::reconnect()
   {
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
 
     if (!options->autoReconnect)
     {
