@@ -22,6 +22,7 @@
 
 #include "logger/LoggerFactory.h"
 #include "Results.h"
+#include "parameters/ParameterHolder.h"
 #include "util/LogQueryTool.h"
 #include "util/ClientPrepareResult.h"
 #include "util/ServerPrepareResult.h"
@@ -46,7 +47,7 @@ namespace capi
 
   static const int64_t MAX_PACKET_LENGTH= 0x00ffffff + 4;
 
-  const Shared::Logger QueryProtocol::logger= LoggerFactory::getLogger(typeid(QueryProtocol));
+  Logger* QueryProtocol::logger= LoggerFactory::getLogger(typeid(QueryProtocol));
   const SQLString QueryProtocol::CHECK_GALERA_STATE_QUERY("show status like 'wsrep_local_state'");
 
   void throwStmtError(MYSQL_STMT* stmt) {
@@ -62,8 +63,8 @@ namespace capi
    * @param urlParser connection URL information's
    * @param lock the lock for thread synchronisation
    */
-  QueryProtocol::QueryProtocol(std::shared_ptr<UrlParser>& urlParser, GlobalStateInfo* globalInfo, Shared::mutex& lock)
-    : super(urlParser, globalInfo, lock)
+  QueryProtocol::QueryProtocol(std::shared_ptr<UrlParser>& urlParser, GlobalStateInfo* globalInfo)
+    : super(urlParser, globalInfo)
     , logQuery(new LogQueryTool(options))
   {
     if (!urlParser->getOptions()->galeraAllowedState.empty())
@@ -81,14 +82,11 @@ namespace capi
       {
         throw SQLException("Connection reset failed");
       }
+      serverPrepareStatementCache->clear();
 
-      if (options->cachePrepStmts && options->useServerPrepStmts){
-        //serverPrepareStatementCache->clear();
-      }
-
-    }catch (SQLException& sqlException){
+    } catch (SQLException& sqlException) {
       throw logQuery->exceptionWithQuery("COM_RESET_CONNECTION failed.", sqlException, explicitClosed);
-    }catch (std::runtime_error& e){
+    } catch (std::runtime_error& e) {
       handleIoException(e).Throw();
     }
   }
@@ -103,18 +101,17 @@ namespace capi
    */
   void QueryProtocol::executeQuery(const SQLString& sql)
   {
-    Shared::Results res(new Results());
-    executeQuery(isMasterConnection(), res, sql);
+    Results res;
+    executeQuery(isMasterConnection(), &res, sql);
   }
 
-
-  void QueryProtocol::executeQuery(bool /*mustExecuteOnMaster*/, Shared::Results& results, const SQLString& sql)
+  void QueryProtocol::executeQuery(bool /*mustExecuteOnMaster*/, Results* results, const SQLString& sql)
   {
     cmdPrologue();
     try {
 
       realQuery(sql);
-      getResult(results.get());
+      getResult(results);
 
     }catch (SQLException& sqlException){
       if (sqlException.getSQLState().compare("70100") == 0 && 1927 == sqlException.getErrorCode()){
@@ -127,13 +124,13 @@ namespace capi
   }
 
 
-  void QueryProtocol::executeQuery( bool /*mustExecuteOnMaster*/, Shared::Results& results, const SQLString& sql, const Charset* /*charset*/)
+  void QueryProtocol::executeQuery( bool /*mustExecuteOnMaster*/, Results* results, const SQLString& sql, const Charset* /*charset*/)
   {
     cmdPrologue();
     try {
 
       realQuery(sql);
-      getResult(results.get());
+      getResult(results);
 
     }catch (SQLException& sqlException){
       throw logQuery->exceptionWithQuery(sql, sqlException, explicitClosed);
@@ -153,13 +150,14 @@ namespace capi
    */
   void QueryProtocol::executeQuery(
       bool mustExecuteOnMaster,
-      Shared::Results& results,
+      Results* results,
       ClientPrepareResult* clientPrepareResult,
-      std::vector<Shared::ParameterHolder>& parameters)
+      std::vector<Unique::ParameterHolder>& parameters)
   {
     executeQuery(mustExecuteOnMaster, results, clientPrepareResult, parameters, -1);
   }
 
+
   SQLString& addQueryTimeout(SQLString& sql, int32_t queryTimeout)
   {
     if (queryTimeout > 0) {
@@ -168,16 +166,37 @@ namespace capi
     return sql;
   }
 
+
+  std::size_t estimatePreparedQuerySize(ClientPrepareResult* clientPrepareResult, const std::vector<std::string> &queryPart,
+      std::vector<Unique::ParameterHolder>& parameters)
+  {
+    std::size_t estimate= queryPart.front().length() + 1/* for \0 */, offset = 0;
+    if (clientPrepareResult->isRewriteType()) {
+      estimate+= queryPart[1].length() + queryPart[clientPrepareResult->getParamCount() + 2].length();
+      offset= 1;
+    }
+    for (uint32_t i = 0; i < clientPrepareResult->getParamCount(); ++i) {
+      estimate+= (parameters)[i]->getApproximateTextProtocolLength();
+      estimate+= queryPart[i + 1 + offset].length();
+    }
+    estimate= ((estimate + 7) / 8) * 8;
+    return estimate;
+  }
+
   void assemblePreparedQueryForExec(
     SQLString& out,
     ClientPrepareResult* clientPrepareResult,
-    std::vector<Shared::ParameterHolder>& parameters,
+    std::vector<Unique::ParameterHolder>& parameters,
     int32_t queryTimeout)
   {
     addQueryTimeout(out, queryTimeout);
 
-    const std::vector<SQLString> &queryPart= clientPrepareResult->getQueryParts();
+    auto &queryPart= clientPrepareResult->getQueryParts();
+    std::size_t estimate= estimatePreparedQuerySize(clientPrepareResult, queryPart, parameters);
 
+    if (estimate > StringImp::get(out).capacity() - out.length()) {
+      out.reserve(out.length() + estimate);
+    }
     if (clientPrepareResult->isRewriteType()) {
 
       out.append(queryPart[0]);
@@ -188,10 +207,8 @@ namespace capi
         out.append(queryPart[i + 2]);
       }
       out.append(queryPart[clientPrepareResult->getParamCount() + 2]);
-
     }
     else {
-
       out.append(queryPart.front());
       for (uint32_t i = 0; i < clientPrepareResult->getParamCount(); i++) {
         parameters[i]->writeTo(out);
@@ -199,6 +216,7 @@ namespace capi
       }
     }
   }
+
   /**
    * Execute a unique clientPrepareQuery.
    *
@@ -211,22 +229,20 @@ namespace capi
    */
   void QueryProtocol::executeQuery(
       bool /*mustExecuteOnMaster*/,
-      Shared::Results& results,
+      Results* results,
       ClientPrepareResult* clientPrepareResult,
-      std::vector<Shared::ParameterHolder>& parameters,
+      std::vector<Unique::ParameterHolder>& parameters,
       int32_t queryTimeout)
   {
     cmdPrologue();
 
     SQLString sql;
-    addQueryTimeout(sql, queryTimeout);
-
     try {
-
+      addQueryTimeout(sql, queryTimeout);
       if (clientPrepareResult->getParamCount() == 0
         && !clientPrepareResult->isQueryMultiValuesRewritable()) {
-        if (clientPrepareResult->getQueryParts().size() == 1) {
-          sql.append(clientPrepareResult->getQueryParts().front());
+        if (clientPrepareResult->getQueryParts().size() == 0) {
+          sql.append(clientPrepareResult->getSql());
           realQuery(sql);
         }
         else {
@@ -242,7 +258,7 @@ namespace capi
         assemblePreparedQueryForExec(sql, clientPrepareResult, parameters, -1);
         realQuery(sql);
       }
-      getResult(results.get());
+      getResult(results);
 
     }
     catch (SQLException& queryException) {
@@ -265,11 +281,10 @@ namespace capi
    */
   bool QueryProtocol::executeBatchClient(
       bool mustExecuteOnMaster,
-      Shared::Results& results,
+      Results* results,
       ClientPrepareResult* prepareResult,
-      std::vector<std::vector<Shared::ParameterHolder>>& parametersList,
+      std::vector<std::vector<Unique::ParameterHolder>>& parametersList,
       bool hasLongData)
-
   {
     // ***********************************************************************************************************
     // Multiple solution for batching :
@@ -280,15 +295,16 @@ namespace capi
     // - one after the other
     // ***********************************************************************************************************
 
-    if (options->rewriteBatchedStatements){
+    if (options->rewriteBatchedStatements) {
       if (prepareResult->isQueryMultiValuesRewritable()
-        && results->getAutoGeneratedKeys() == Statement::NO_GENERATED_KEYS){
+        && results->getAutoGeneratedKeys() == Statement::NO_GENERATED_KEYS) {
         // values rewritten in one query :
         // INSERT INTO X(a,b) VALUES (1,2), (3,4), ...
         executeBatchRewrite(results, prepareResult, parametersList, true);
         return true;
 
-      }else if (prepareResult->isQueryMultipleRewritable()){
+      }
+      else if (prepareResult->isQueryMultipleRewritable()) {
 
         if (options->useBulkStmts
             && !hasLongData
@@ -308,7 +324,7 @@ namespace capi
     if (options->useBulkStmts
         && !hasLongData
         && results->getAutoGeneratedKeys()==Statement::NO_GENERATED_KEYS
-        && executeBulkBatch(results,prepareResult->getSql(),nullptr,parametersList)){
+        && executeBulkBatch(results, prepareResult->getSql(), nullptr, parametersList)) {
       return true;
     }
 
@@ -333,9 +349,9 @@ namespace capi
    * @throws SQLException exception
    */
   bool QueryProtocol::executeBulkBatch(
-      Shared::Results& results, const SQLString& origSql,
+      Results* results, const SQLString& origSql,
       ServerPrepareResult* serverPrepareResult,
-      std::vector<std::vector<Shared::ParameterHolder>>& parametersList)
+      std::vector<std::vector<Unique::ParameterHolder>>& parametersList)
   {
     const int16_t NullType= ColumnType::_NULL.getType();
     // **************************************************************************************
@@ -350,7 +366,7 @@ namespace capi
       return false;
 
     // ensure that type doesn't change
-    std::vector<Shared::ParameterHolder> initParameters= parametersList.front();
+    std::vector<Unique::ParameterHolder> &initParameters= parametersList.front();
     std::size_t parameterCount= initParameters.size();
     std::vector<int16_t> types;
     types.reserve(parameterCount);
@@ -397,9 +413,6 @@ namespace capi
         tmpServerPrepareResult= prepareInternal(origSql, true);
       }
 
-      // **************************************************************************************
-      // send BULK
-      // **************************************************************************************
       capi::MYSQL_STMT* statementId= tmpServerPrepareResult ? tmpServerPrepareResult->getStatementId() : nullptr;
 
       //TODO: shouldn't throw if stmt is NULL? Returning false so far.
@@ -410,22 +423,24 @@ namespace capi
       unsigned int bulkArrSize= static_cast<unsigned int>(parametersList.size());
 
       capi::mysql_stmt_attr_set(statementId, STMT_ATTR_ARRAY_SIZE, (const void*)&bulkArrSize);
-      auto firstParameters= parametersList.front();
 
       tmpServerPrepareResult->bindParameters(parametersList, types.data());
+      // **************************************************************************************
+      // send BULK
+      // **************************************************************************************
       capi::mysql_stmt_execute(statementId);
 
       try {
-        getResult(results.get(), tmpServerPrepareResult);
+        getResult(results, tmpServerPrepareResult);
       }
       catch (SQLException& sqle) {
         if (!serverPrepareResult && tmpServerPrepareResult) {
-          //releasePrepareStatement(tmpServerPrepareResult);
+          releasePrepareStatement(tmpServerPrepareResult);
           // releasePrepareStatement basically cares only about releasing stmt on server(and C API handle)
           delete tmpServerPrepareResult;
           tmpServerPrepareResult= nullptr;
         }
-        if (sqle.getSQLState().compare("HY000") == 0 && sqle.getErrorCode()==1295) {
+        if (sqle.getSQLState().compare("HY000") == 0 && sqle.getErrorCode()==1295){
           // query contain commands that cannot be handled by BULK protocol
           // clear error and special error code, so it won't leak anywhere
           // and wouldn't be misinterpreted as an additional update count
@@ -440,21 +455,22 @@ namespace capi
         }
       }
 
-      if (!exception.getMessage().empty()) {
-        throw exception;
-      }
       results->setRewritten(true);
       
       if (!serverPrepareResult && tmpServerPrepareResult) {
-        //releasePrepareStatement(tmpServerPrepareResult);
+        releasePrepareStatement(tmpServerPrepareResult);
         // releasePrepareStatement basically cares only about releasing stmt on server(and C API handle)
         delete tmpServerPrepareResult;
       }
+      
+      if (!exception.getMessage().empty()) {
+        throw exception;
+      }
       return true;
     }
     catch (std::runtime_error& e) {
       if (!serverPrepareResult && tmpServerPrepareResult) {
-        //releasePrepareStatement(tmpServerPrepareResult);
+        releasePrepareStatement(tmpServerPrepareResult);
         // releasePrepareStatement basically cares only about releasing stmt on server(and C API handle)
         delete tmpServerPrepareResult;
       }
@@ -464,6 +480,7 @@ namespace capi
     return false;
   }
 
+
   void QueryProtocol::initializeBatchReader()
   {
     if (options->useBatchMultiSend){
@@ -480,10 +497,9 @@ namespace capi
    * @throws SQLException exception
    */
   void QueryProtocol::executeBatchMulti(
-      Shared::Results& results,
+      Results* results,
       ClientPrepareResult* clientPrepareResult,
-      std::vector<std::vector<Shared::ParameterHolder>>& parametersList)
-
+      std::vector<std::vector<Unique::ParameterHolder>>& parametersList)
   {
     cmdPrologue();
     initializeBatchReader();
@@ -513,7 +529,7 @@ namespace capi
     for (std::size_t i= 0; i < parametersList.size(); ++i) {
       // We don't need exception on error here
       capi::mysql_read_query_result(connection.get());
-      getResult(results.get());
+      getResult(results);
     }
     if (autoCommit) {
       // Getting result for commit and setting autocommit back on to clear the connection,
@@ -532,9 +548,9 @@ namespace capi
    */
   void QueryProtocol::executeBatchSlow(
     bool mustExecuteOnMaster,
-    Shared::Results& results,
+    Results* results,
     ClientPrepareResult* clientPrepareResult,
-    std::vector<std::vector<Shared::ParameterHolder>>& parametersList)
+    std::vector<std::vector<Unique::ParameterHolder>>& parametersList)
   {
     cmdPrologue();
     // send query one by one, reading results for each query before sending another one
@@ -581,7 +597,7 @@ namespace capi
    * @param queries queries
    * @throws SQLException if any exception occur
    */
-  void QueryProtocol::executeBatchStmt(bool /*mustExecuteOnMaster*/, Shared::Results& results, const std::vector<SQLString>& queries)
+  void QueryProtocol::executeBatchStmt(bool /*mustExecuteOnMaster*/, Results* results, const std::vector<SQLString>& queries)
   {
     cmdPrologue();
     if (this->options->rewriteBatchedStatements) {
@@ -589,7 +605,7 @@ namespace capi
       // check that queries are rewritable
       bool canAggregateSemiColumn= true;
       std::size_t totalLen= 0;
-      for (SQLString query : queries){
+      for (auto& query : queries){
         if (!ClientPrepareResult::canAggregateSemiColon(query,noBackslashEscapes())){
           canAggregateSemiColumn= false;
           break;
@@ -621,7 +637,7 @@ namespace capi
    * @param queries list of queries
    * @throws SQLException exception
    */
-  void QueryProtocol::executeBatch(Shared::Results& results, const std::vector<SQLString>& queries)
+  void QueryProtocol::executeBatch(Results* results, const std::vector<SQLString>& queries)
   {
     bool autoCommit= getAutocommit();
     
@@ -633,7 +649,7 @@ namespace capi
         try {
           stopIfInterrupted();
           realQuery(sql);
-          getResult(results.get());
+          getResult(results);
         }
         catch (SQLException& sqlException) {
           SQLException ex(logQuery->exceptionWithQuery(sql, sqlException, explicitClosed));
@@ -673,7 +689,7 @@ namespace capi
     for (auto& query : queries) {
       //we don't need exception in case of error, thus calling capi directly
       capi::mysql_read_query_result(connection.get());
-      getResult(results.get());
+      getResult(results);
     }
     if (autoCommit) {
       commitReturnAutocommit(true);
@@ -683,13 +699,11 @@ namespace capi
 
   ServerPrepareResult* QueryProtocol::prepareInternal(const SQLString& sql, bool /*executeOnMaster*/)
   {
-    if (options->cachePrepStmts && options->useServerPrepStmts) {
+    const SQLString key(getDatabase() + "-" + sql);
+    ServerPrepareResult* pr= serverPrepareStatementCache->get(StringImp::get(key));
 
-      ServerPrepareResult* pr = serverPrepareStatementCache->get(database + "-" + sql);
-
-      if (pr && pr->incrementShareCounter()) {
-        return pr;
-      }
+    if (pr) {
+       return pr;
     }
 
     capi::MYSQL_STMT* stmtId = capi::mysql_stmt_init(connection.get());
@@ -712,22 +726,21 @@ namespace capi
       throw SQLException(err, sqlState, errNo);
     }
 
-    ServerPrepareResult* res= new ServerPrepareResult(sql, stmtId, this);
+    pr= new ServerPrepareResult(sql, stmtId, this);
 
-    if (options->cachePrepStmts
-      && options->useServerPrepStmts
-      && sql.length() < static_cast<size_t>(options->prepStmtCacheSqlLimit)) {
-      SQLString key(getDatabase() + "-" + sql);
+    // This condition is now enforced on cache level on the key length
+    //if (sql.length() < static_cast<size_t>(options->prepStmtCacheSqlLimit))
+    {
 
-      ServerPrepareResult* cachedServerPrepareResult= addPrepareInCache(key, res);
+      ServerPrepareResult* cachedServerPrepareResult= addPrepareInCache(key, pr);
 
       if (cachedServerPrepareResult != nullptr)
       {
-        delete res;
-        res= cachedServerPrepareResult;
+        delete pr;
+        pr= cachedServerPrepareResult;
       }
     }
-    return res;
+    return pr;
   }
 
 
@@ -759,7 +772,6 @@ namespace capi
       sql.append(';').append(queries[currentIndex]);
       ++currentIndex;
     }
-
     return currentIndex;
   }
 
@@ -772,9 +784,8 @@ namespace capi
    * @param queries list of queries
    * @throws SQLException exception
    */
-  void QueryProtocol::executeBatchAggregateSemiColon(Shared::Results& results, const std::vector<SQLString>& queries, std::size_t totalLenEstimation)
+  void QueryProtocol::executeBatchAggregateSemiColon(Results* results, const std::vector<SQLString>& queries, std::size_t totalLenEstimation)
   {
-
     SQLString firstSql;
     size_t currentIndex= 0;
     size_t totalQueries= queries.size();
@@ -789,33 +800,34 @@ namespace capi
         if (totalLenEstimation == 0) {
           totalLenEstimation= firstSql.length()*queries.size() + queries.size() - 1;
         }
-        sql.reserve(((std::min<int64_t>(MAX_PACKET_LENGTH, totalLenEstimation) + 7) / 8) * 8);
+        sql.reserve(((std::min<int64_t>(MAX_PACKET_LENGTH, static_cast<int64_t>(totalLenEstimation)) + 7) / 8) * 8);
         currentIndex= assembleBatchAggregateSemiColonQuery(sql, firstSql, queries, currentIndex);
         realQuery(sql);
         sql.clear(); // clear is not supposed to release memory
 
-        getResult(results.get(), nullptr, true);
+        getResult(results, nullptr, true);
 
-      }catch (SQLException& sqlException){
+      }
+      catch (SQLException& sqlException) {
         if (exception.getMessage().empty()){
           exception= logQuery->exceptionWithQuery(firstSql, sqlException, explicitClosed);
           if (!options->continueBatchOnError){
             throw exception;
           }
         }
-      }catch (std::runtime_error& e){
+      }
+      catch (std::runtime_error& e) {
         handleIoException(e).Throw();
       }
       stopIfInterrupted();
 
-    }while (currentIndex < totalQueries);
+    } while (currentIndex < totalQueries);
 
     if (!exception.getMessage().empty()) {
       throw exception;
     }
   }
 
-
   /**
   * Client side PreparedStatement.executeBatch values rewritten (concatenate value params according
   * to max_allowed_packet)
@@ -830,18 +842,17 @@ namespace capi
   * @throws IOException if connection fail
   */
   std::size_t rewriteQuery(SQLString& pos,
-    const std::vector<SQLString> &queryParts,
+    const std::vector<std::string> &queryParts,
     std::size_t currentIndex,
     std::size_t paramCount,
-    std::vector<std::vector<Shared::ParameterHolder>> &parameterList,
+    std::vector<std::vector<Unique::ParameterHolder>>& parameterList,
     bool rewriteValues)
-
   {
-    std::size_t index= currentIndex;
-    std::vector<Shared::ParameterHolder> &parameters= parameterList[index++];
+    std::size_t index= currentIndex, capacity= StringImp::get(pos).capacity(), estimatedLength;
+    std::vector<Unique::ParameterHolder> &parameters= parameterList[index++];
 
     const SQLString &firstPart= queryParts[1];
-    const SQLString &secondPart= queryParts[0];
+    const SQLString &secondPart= queryParts.front();
 
     if (!rewriteValues) {
 
@@ -855,17 +866,20 @@ namespace capi
 
       for (size_t i= 0; i < paramCount; i++) {
         parameters[i]->writeTo(pos);
-        pos.append(queryParts[i +2]);
+        pos.append(queryParts[i + 2]);
       }
-      pos.append(queryParts[paramCount +2]);
-
-
-      while (index <parameterList.size()) {
-        parameters= parameterList[index];
+      pos.append(queryParts[paramCount + 2]);
 
+      estimatedLength= pos.length() * (parameterList.size() - currentIndex);
+      if (estimatedLength > capacity) {
+        pos.reserve(((std::min<int64_t>(MAX_PACKET_LENGTH, static_cast<int64_t>(estimatedLength)) + 7) / 8) * 8);
+      }
 
+      while (index < parameterList.size()) {
+        auto& parameters= parameterList[index];
         int64_t parameterLength= 0;
         bool knownParameterSize= true;
+
         for (auto& parameter : parameters) {
           int64_t paramSize= parameter->getApproximateTextProtocolLength();
           if (paramSize == -1) {
@@ -877,7 +891,6 @@ namespace capi
 
         if (knownParameterSize) {
 
-
           if (checkRemainingSize(pos.length() + staticLength + parameterLength)) {
             pos.append(';');
             pos.append(firstPart);
@@ -886,7 +899,7 @@ namespace capi
               parameters[i]->writeTo(pos);
               pos.append(queryParts[i + 2]);
             }
-            pos.append(queryParts[paramCount +2]);
+            pos.append(queryParts[paramCount + 2]);
             ++index;
           }
           else {
@@ -902,7 +915,7 @@ namespace capi
             parameters[i]->writeTo(pos);
             pos.append(queryParts[i +2]);
           }
-          pos.append(queryParts[paramCount +2]);
+          pos.append(queryParts[paramCount + 2]);
           ++index;
           break;
         }
@@ -912,7 +925,7 @@ namespace capi
     else {
       pos.append(firstPart);
       pos.append(secondPart);
-      size_t lastPartLength= queryParts[paramCount +2].length();
+      size_t lastPartLength= queryParts[paramCount + 2].length();
       size_t intermediatePartLength= queryParts[1].length();
 
       for (size_t i= 0; i <paramCount; i++) {
@@ -921,9 +934,8 @@ namespace capi
         intermediatePartLength +=queryParts[i +2].length();
       }
 
-      while (index <parameterList.size()) {
-        parameters= parameterList[index];
-
+      while (index < parameterList.size()) {
+        auto& parameters= parameterList[index];
 
         int64_t parameterLength= 0;
         bool knownParameterSize= true;
@@ -938,7 +950,6 @@ namespace capi
 
         if (knownParameterSize) {
 
-
           if (checkRemainingSize(pos.length() + 1 + parameterLength + intermediatePartLength + lastPartLength)) {
             pos.append(',');
             pos.append(secondPart);
@@ -981,9 +992,9 @@ namespace capi
    * @throws SQLException exception
    */
   void QueryProtocol::executeBatchRewrite(
-      Shared::Results& results,
+      Results* results,
       ClientPrepareResult* prepareResult,
-      std::vector<std::vector<Shared::ParameterHolder>>& parameterList,
+      std::vector<std::vector<Unique::ParameterHolder>>& parameterList,
       bool rewriteValues)
   {
     cmdPrologue();
@@ -998,7 +1009,7 @@ namespace capi
         sql.clear();
         currentIndex= rewriteQuery(sql, prepareResult->getQueryParts(), currentIndex, prepareResult->getParamCount(), parameterList, rewriteValues);
         realQuery(sql);
-        getResult(results.get(), nullptr, !rewriteValues);
+        getResult(results, nullptr, !rewriteValues);
 
 #ifdef THE_TIME_HAS_COME
         if (Thread.currentThread().isInterrupted()){
@@ -1008,13 +1019,14 @@ namespace capi
 #endif
       } while (currentIndex < totalParameterList);
 
-    }catch (SQLException& sqlEx){
-      throw logQuery->exceptionWithQuery(sqlEx,prepareResult);
-    }catch (std::runtime_error& e){
-      handleIoException(e).Throw();
-    }/* TODO: something with the finally was once here */ {
+    } catch (SQLException& sqlEx) {
       results->setRewritten(rewriteValues);
+      throw logQuery->exceptionWithQuery(sqlEx,prepareResult);
+    } catch (std::runtime_error& e) {
+      results->setRewritten(rewriteValues);
+      handleIoException(e).Throw();
     }
+    results->setRewritten(rewriteValues);
   }
 
   /**
@@ -1032,8 +1044,8 @@ namespace capi
   bool QueryProtocol::executeBatchServer(
       bool /*mustExecuteOnMaster*/,
       ServerPrepareResult* serverPrepareResult,
-      Shared::Results& results, const SQLString& sql,
-      std::vector<std::vector<Shared::ParameterHolder>>& parametersList,
+      Results* results, const SQLString& sql,
+      std::vector<std::vector<Unique::ParameterHolder>>& parametersList,
       bool hasLongData)
   {
     bool needToRelease= false;
@@ -1070,8 +1082,8 @@ namespace capi
   void QueryProtocol::executePreparedQuery(
       bool /*mustExecuteOnMaster*/,
       ServerPrepareResult* serverPrepareResult,
-      Shared::Results& results,
-      std::vector<Shared::ParameterHolder>& parameters)
+      Results* results,
+      std::vector<Unique::ParameterHolder>& parameters)
   {
     cmdPrologue();
 
@@ -1103,9 +1115,11 @@ namespace capi
         throwStmtError(serverPrepareResult->getStatementId());
       }
       /*CURSOR_TYPE_NO_CURSOR);*/
-      getResult(results.get(), serverPrepareResult);
-      // We have to do this due to CONCPP-138
-      results->loadFully(false, this);
+      getResult(results, serverPrepareResult);
+      // We have to do this due to CONCPP-138, but only when we are not streaming
+      if (results->getFetchSize() == 0) {
+        results->loadFully(false, this);
+      }
     }
     catch (SQLException& qex) {
       throw logQuery->exceptionWithQuery(parameters, qex, serverPrepareResult);
@@ -1120,13 +1134,11 @@ namespace capi
   {
     cmdPrologue();
 
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
     try {
-
       if (inTransaction()){
         executeQuery("ROLLBACK");
       }
-
     } catch (std::runtime_error&){
     }
   }
@@ -1139,24 +1151,23 @@ namespace capi
    * @return true if successfully released
    * @throws SQLException if connection exception.
    */
-  bool QueryProtocol::forceReleasePrepareStatement(MYSQL_STMT* statementId)
+  bool QueryProtocol::forceReleasePrepareStatement(capi::MYSQL_STMT* statementId)
   {
-
-    if (lock->try_lock()) {
-      checkClose();
-
+    // We need lock only if still connected. if we get disconnected after we read "connected" flag - still no big deal
+    bool needLock= connected;
+    if (!needLock || lock.try_lock()) {
+      // If connection is closed - we still need to call mysql_stmt_close to deallocate C/C stmt handle
       if (mysql_stmt_close(statementId))
       {
-        connected= false;
-        lock->unlock();
+        needLock && (lock.unlock(), true);
         throw SQLException(
             "Could not deallocate query",
             CONNECTION_EXCEPTION.getSqlState().c_str());
       }
-      lock->unlock();
+      needLock && (lock.unlock(), true);
       return true;
-
-    }else {
+    }
+    else {
       statementIdToRelease= statementId;
     }
 
@@ -1179,9 +1190,8 @@ namespace capi
 
   bool QueryProtocol::ping()
   {
-
     cmdPrologue();
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
     try {
 
       return mysql_ping(connection.get()) == 0;
@@ -1195,20 +1205,19 @@ namespace capi
 
   bool QueryProtocol::isValid(int32_t timeout)
   {
-
     int32_t initialTimeout= -1;
+
     try {
       initialTimeout= this->socketTimeout;
-      if (initialTimeout == 0){
+      if (initialTimeout == 0) {
         this->changeSocketSoTimeout(timeout);
       }
-      if (isMasterConnection() && galeraAllowedStates && galeraAllowedStates->size() != 0){
+      if (isMasterConnection() && galeraAllowedStates && galeraAllowedStates->size() != 0) {
 
-
-        Shared::Results results(new Results());
-        executeQuery(true, results, CHECK_GALERA_STATE_QUERY);
-        results->commandEnd();
-        ResultSet* rs= results->getResultSet();
+        Results results;
+        executeQuery(true, &results, CHECK_GALERA_STATE_QUERY);
+        results.commandEnd();
+        ResultSet* rs= results.getResultSet();
 
         if (rs && rs->next())
         {
@@ -1228,8 +1237,8 @@ namespace capi
       }
 
       return ping();
-
-    }catch (/*SocketException*/std::runtime_error& socketException){
+    }
+    catch (/*SocketException*/std::runtime_error& socketException){
       logger->trace(SQLString("Connection* is not valid").append(socketException.what()));
       connected= false;
       return false;
@@ -1249,22 +1258,21 @@ namespace capi
     }*/
   }
 
+
   SQLString QueryProtocol::getCatalog()
   {
-
-    if ((serverCapabilities & MariaDbServerCapabilities::CLIENT_SESSION_TRACK)!=0){
-
+    if ((serverCapabilities & MariaDbServerCapabilities::CLIENT_SESSION_TRACK) != 0) {
       return database;
     }
 
     cmdPrologue();
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
 
-    Shared::Results results(new Results());
-    executeQuery(isMasterConnection(), results, "select database()");
-    results->commandEnd();
-    ResultSet* rs= results->getResultSet();
-    if (rs->next()){
+    Results results;
+    executeQuery(isMasterConnection(), &results, "SELECT DATABASE()");
+    results.commandEnd();
+    ResultSet* rs= results.getResultSet();
+    if (rs->next()) {
       this->database= rs->getString(1);
       return database;
     }
@@ -1277,7 +1285,7 @@ namespace capi
 
     cmdPrologue();
 
-    std::unique_lock<std::mutex> localScopeLock(*lock);
+    std::unique_lock<std::mutex> localScopeLock(lock);
 
     if (capi::mysql_select_db(connection.get(), _database.c_str()) != 0) {
       // TODO: realQuery should throw. Here we could catch and change message
@@ -1307,9 +1315,7 @@ namespace capi
 
   void QueryProtocol::cancelCurrentQuery()
   {
-    Shared::mutex newMutex(new std::mutex());
-    std::unique_ptr<MasterProtocol> copiedProtocol(
-      new MasterProtocol(urlParser, new GlobalStateInfo(), newMutex));
+    std::unique_ptr<MasterProtocol> copiedProtocol(new MasterProtocol(urlParser, new GlobalStateInfo()));
 
     copiedProtocol->setHostAddress(getHostAddress());
     copiedProtocol->connect();
@@ -1332,10 +1338,17 @@ namespace capi
 
   void QueryProtocol::closeExplicit()
   {
+    GET_LOGGER()->trace("Protocol::closeExplicit:", std::hex, this);
     this->explicitClosed= true;
     close();
   }
 
+  void QueryProtocol::markClosed(bool closed)
+  {
+    GET_LOGGER()->trace("Protocol::markClosed:", std::hex, this, closed);
+    this->explicitClosed = closed;
+    //close();
+  }
 
   void QueryProtocol::releasePrepareStatement(ServerPrepareResult* serverPrepareResult)
   {
@@ -1368,12 +1381,12 @@ namespace capi
     }
   }
 
-
+#ifdef WE_DO_OWN_PROTOCOL_IMPEMENTATION
   void QueryProtocol::setLocalInfileInputStream(std::istream& inputStream)
   {
     this->localInfileInputStream.reset(&inputStream);
   }
-
+#endif
 
   int32_t QueryProtocol::getTimeout()
   {
@@ -1383,7 +1396,7 @@ namespace capi
 
   void QueryProtocol::setTimeout(int32_t timeout)
   {
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
 
     this->changeSocketSoTimeout(timeout);
   }
@@ -1397,7 +1410,7 @@ namespace capi
   void QueryProtocol::setTransactionIsolation(int32_t level)
   {
     cmdPrologue();
-    std::lock_guard<std::mutex> localScopeLock(*lock);
+    std::lock_guard<std::mutex> localScopeLock(lock);
 
     SQLString query= "SET SESSION TRANSACTION ISOLATION LEVEL";
 
@@ -1418,7 +1431,7 @@ namespace capi
         throw SQLException("Unsupported transaction isolation level");
     }
 
-    executeQuery(query);
+    realQuery(query);
     transactionIsolationLevel= level;
   }
 
@@ -1579,16 +1592,17 @@ namespace capi
   int32_t QueryProtocol::getAutoIncrementIncrement()
   {
     if (autoIncrementIncrement == 0) {
-      std::lock_guard<std::mutex> localScopeLock(*lock);
+      std::lock_guard<std::mutex> localScopeLock(lock);
       try {
-        Shared::Results results(new Results());
-        executeQuery(true, results,"select @@auto_increment_increment");
-        results->commandEnd();
-        ResultSet* rs= results->getResultSet();
+        Results results;
+        executeQuery(true, &results, "SELECT @@auto_increment_increment");
+        results.commandEnd();
+        ResultSet* rs= results.getResultSet();
         rs->next();
         autoIncrementIncrement= rs->getInt(1);
-      }catch (SQLException& e){
-        if (e.getSQLState().startsWith("08")){
+      }
+      catch (SQLException& e) {
+        if (e.getSQLState().startsWith("08")) {
           throw e;
         }
         autoIncrementIncrement= 1;
@@ -1639,7 +1653,7 @@ namespace capi
   void QueryProtocol::readLocalInfilePacket(Shared::Results& /*results*/)
   {
 
-#ifdef UNLIKELY_WE_HAVE_TO_DO_ANYTHING_HERE_CARED_BY_CAPI
+#ifdef WE_DO_OWN_PROTOCOL_IMPEMENTATION
     int32_t seq= 2;
     SQLString fileName= buffer->readStringNullEnd(StandardCharsets.UTF_8);
     try {
@@ -1769,13 +1783,13 @@ namespace capi
           selectResultSet= UpdatableResultSet::create(results, this, pr, callableResult, eofDeprecated);
         }
       }
+
       // Not sure where we get status and more results there is and if it's available if we are streaming result
       bool pendingResults= hasMoreResults() || results->getFetchSize() > 0;
       results->addResultSet(selectResultSet, pendingResults);
       if (pendingResults) {
         setActiveStreamingResult(results);
       }
-
     }
     catch (SQLException & e) {
       throw e;
@@ -1831,7 +1845,7 @@ namespace capi
 
   ServerPrepareResult* QueryProtocol::addPrepareInCache(const SQLString& key, ServerPrepareResult* serverPrepareResult)
   {
-    return serverPrepareStatementCache->put(key, serverPrepareResult);
+    return serverPrepareStatementCache->put(StringImp::get(key), serverPrepareResult);
   }
 
   void QueryProtocol::cmdPrologue()
@@ -1842,8 +1856,7 @@ namespace capi
       activeStreamingResult= nullptr;
     }
     forceReleaseWaitingPrepareStatement();
-    if (activeFutureTask){
-
+    if (activeFutureTask) {
       try {
         //activeFutureTask->get();
       }
@@ -1859,7 +1872,6 @@ namespace capi
             interruptedException);
       }*/
       /*finally*/
-
       activeFutureTask= nullptr;
     }
 
