@@ -1947,10 +1947,11 @@ void connection::connectOptReconnect()
 {
   logMsg("connection::connectOptReconnect - OPT_RECONNECT");
   std::stringstream msg;
-    if (isMaxScale()) {
-      printf("# <<<<  canceled for maxscale \n# ");
-      return void();
-    }
+
+  if (isMaxScale()) {
+    printf("# <<<<  canceled for maxscale \n# ");
+    return void();
+  }
 
   try
   {
@@ -3284,12 +3285,45 @@ void connection::useCharacterSet()
   }
 }
 
+/* Using 1.0 API types with 1.1 driver. i.e. Properties was just a map, now it's defined in the driver class.
+   getTables used std::list as parameter type.
+   We need this test to be compiled und do not crash */
+void connection::concpp89_10to11upgrade()
+{
+  typedef std::map<sql::SQLString, sql::SQLString> Properties1_0;
+  typedef std::list< sql::SQLString > getTablesListParam1_0;
+  Properties1_0 p;
+
+  p["user"] = user;
+  p["password"] = passwd;
+  p["useTls"] = useTls ? "true" : "false";
+
+  con.reset(driver->connect(url, p));
+  ASSERT(con.get());
+
+  p["hostName"] = url;
+  con.reset(driver->connect(p));
+  ASSERT(con.get());
+
+  con.reset(sql::DriverManager::getConnection(url, p));
+
+  // Also getTables parameter type has been changed. Testing it still can be compiled with parameter type from 1.0
+  getTablesListParam1_0 tableTypes;
+  DatabaseMetaData  dbmeta(con->getMetaData());
+
+  tableTypes.push_back("TABLE");
+  res.reset(dbmeta->getTables("", "%", "%", tableTypes));
+
+  res.reset();
+}
+
 
 void connection::concpp94_loadLocalInfile()
 {
   sql::Properties p;
   sql::SQLString onServer(getVariableValue("local_infile", true));
 
+  // Turning it on it it is off at the server
   if (onServer.compare("0") == 0) {
     try {
       setVariableValue("local_infile", "ON", true); // can't be session
@@ -3298,11 +3332,22 @@ void connection::concpp94_loadLocalInfile()
       SKIP("local_infile is OFF at the server, and test could not change that. Doesn't make sense to continue the test");
     }
   }
+  // Assuming allowLocalInfile is off(that is default value)
+  try {
+    stmt->execute("LOAD DATA LOCAL INFILE 'nonexistent.txt' INTO TABLE nonexistent(b)");
+  }
+  catch (sql::SQLException& e) {
+    if (e.getErrorCode() != 1148 && e.getErrorCode() != 4166 &&
+      e.getErrorCode() != 3948) {
+      FAIL("Wrong error code - local infile is allowed");
+    }
+    ASSERT_EQUALS("42000", e.getSQLState());
+  }
 
-  p["user"] = user;
-  p["password"] = passwd;
-  p["allowLocalInfile"] = "true";
-  p["useTls"] = useTls ? "true" : "false";
+  p["user"]=             user;
+  p["password"]=         passwd;
+  p["allowLocalInfile"]= "true";
+  p["useTls"]=           useTls ? "true" : "false";
 
   con.reset(driver->connect(url, p));
   ASSERT(con.get());
@@ -3313,16 +3358,50 @@ void connection::concpp94_loadLocalInfile()
   }
   catch (sql::SQLException& e) {
     if (e.getErrorCode() == 1148 || e.getErrorCode() == 4166) {
-      printf("\n# ERR: Caught sql::SQLException at ::%d  [%s] (%d/%s)\n", __LINE__, e.what(), e.getErrorCode(), e.getSQLStateCStr());
-      printf("# ");
       FAIL("Wrong error code - local infile is still not allowed");
     }
-    //ASSERT(4166!=e.getErrorCode());
     ASSERT_EQUALS("42S02", e.getSQLState());
   }
 }
 
 
+void connection::concpp4_sequentialfailover()
+{
+  const sql::SQLString dummyHost("240.0.0.1:3307"), sequentialPrefix("jdbc:mariadb:sequential://"), hostsSeparator(",");
+  sql::SQLString localUrl(sequentialPrefix + dummyHost + hostsSeparator), realHost, theRest;
+
+  std::size_t doubleSlash= url.find("//"), slash;
+  
+  if (doubleSlash == std::string::npos) {
+    slash= url.find_first_of('/');
+    realHost = url.substr(0, slash);
+  }
+  else {
+    slash= url.find_first_of('/', doubleSlash + 2);
+    realHost = url.substr(doubleSlash + 2, slash - doubleSlash - 2);
+  }
+  theRest= url.substr(slash);
+  localUrl.append(realHost).append(theRest);
+
+  sql::Properties p{{"connectTimeout", "6000"}, {"user", user}, {"password", passwd}, {"useTls", useTls ? "true" : "false"}};
+
+  con.reset(driver->connect(localUrl, p));
+  ASSERT(con.get());
+  stmt.reset(con->createStatement());
+  res.reset(stmt->executeQuery("SELECT CONNECTION_ID()"));
+  ASSERT(res->next());
+
+  //Now good host first
+  localUrl= sequentialPrefix + realHost + hostsSeparator + dummyHost + theRest;
+
+  con.reset(driver->connect(localUrl, p));
+  ASSERT(con.get());
+  stmt.reset(con->createStatement());
+  res.reset(stmt->executeQuery("SELECT CONNECTION_ID()"));
+  ASSERT(res->next());
+}
+
+
 void connection::concpp105_conn_concurrency()
 {
   sql::Properties p{{"user", user}, {"password", passwd}};
@@ -3444,6 +3523,25 @@ void connection::concpp112_connection_attributes()
   con.reset();
 }
 
+
+void connection::concpp146_initSQL()
+{
+  sql::Properties p{{"user", user}, {"password", passwd}};
+  con.reset(driver->connect(addOptions2url("initSql=SET @myVar='YourVar'"), p));
+  stmt.reset(con->createStatement());
+  res.reset(stmt->executeQuery("SELECT @myVar"));
+  ASSERT(res->next());
+  ASSERT_EQUALS("YourVar", res->getString(1));
+
+  con.reset(driver->connect(addOptions2url("&initSql=SET @myVar='YourVar';SET @myVar2='YourVar2'"), p));
+  stmt.reset(con->createStatement());
+  res.reset(stmt->executeQuery("SELECT @myVar, @myVar2"));
+  ASSERT(res->next());
+  ASSERT_EQUALS("YourVar", res->getString(1));
+  ASSERT_EQUALS("YourVar2", res->getString(2));
+}
+
+
 void connection::setUp()
 {
   super::setUp();
