@@ -45,25 +45,19 @@ namespace classes
 {
 void preparedstatement::setUp()
 {
-  sql::SQLString sspsOptionValue;
-  sql::Properties::iterator useServerPrepStmts= commonProperties.find("useServerPrepStmts");
-  bool hadOption= useServerPrepStmts != commonProperties.end();
-
-  if (hadOption) {
-    sspsOptionValue= useServerPrepStmts->second;
-  }
-
   commonProperties["useServerPrepStmts"]= "false";
   super::setUp();
 
   commonProperties["useServerPrepStmts"] = "true";
   try
   {
+    // We do not reset ssps to test it can be safely destroyed after destroying (createdd in previous test) connection
+    // ssps.reset();
     sspsCon.reset(this->getConnection(&commonProperties));
   }
   catch (sql::SQLException& sqle)
   {
-    logErr(String("Couldn't get connection") + sqle.what());
+    logErr(String("Couldn't get connection: ") + sqle.what());
     throw sqle;
   }
 
@@ -1380,6 +1374,9 @@ void preparedstatement::callSP2()
       return;
     }
     res->close();
+    /*while (cstmt->getMoreResults())
+    {
+    }*/
   }
   catch (sql::SQLException &e)
   {
@@ -1852,6 +1849,14 @@ void preparedstatement::executeQuery()
 
 void preparedstatement::addBatch()
 {
+  sql::ConnectOptionsMap connection_properties{{"userName", user},
+                                               {"password", passwd},
+                                               {"useBulkStmts", "false"},
+                                               {"useTls", useTls ? "true" : "false"}};
+
+  con.reset(driver->connect(url, connection_properties));
+  stmt.reset(con->createStatement());
+
   createSchemaObject("TABLE", "testAddBatchPs", "(id int not NULL)");
 
   pstmt.reset(con->prepareStatement("INSERT INTO testAddBatchPs VALUES(?)"));
@@ -1862,7 +1867,7 @@ void preparedstatement::addBatch()
   pstmt->setInt(1, 3);
   pstmt->addBatch();
 
-  const sql::Ints& batchRes = pstmt->executeBatch();
+  const sql::Ints& batchRes= pstmt->executeBatch();
   // Checking results in other connection since connector may turn autocommit off and we have to be sure
   // that it still commits the batch
   stmt.reset(sspsCon->createStatement());
@@ -1908,6 +1913,45 @@ void preparedstatement::addBatch()
   ASSERT_EQUALS(1LL, batchLRes[2]);
 }
 
+
+void preparedstatement::concpp88()
+{
+  pstmt.reset(con->prepareStatement("SELECT 1"));
+  res.reset(pstmt->executeQuery());
+
+  ResultSet rs2(pstmt->executeQuery());
+
+  // Re-execution should close RS from previous execution
+  ASSERT(res->isClosed());
+  pstmt.reset();
+  // Parent statemetne destruction should close RS
+  ASSERT(res->isClosed());
+
+  createSchemaObject("PROCEDURE", "p_conncpp88", "(OUT outparam int) BEGIN SELECT 2; SELECT 1; SELECT 3 INTO outparam; END;");
+  pstmt.reset(con->prepareStatement("CALL p_conncpp88(@resholder)"));
+  rs2.reset(pstmt->executeQuery());
+  ASSERT(pstmt->getMoreResults());
+  ASSERT(rs2->isClosed());
+  res.reset(pstmt->getResultSet());
+  ASSERT(rs2->isClosed());
+  ASSERT(!pstmt->getMoreResults());
+  ASSERT(res->isClosed());
+
+  cstmt.reset(con->prepareCall("CALL p_conncpp88(@resholder)"));
+  rs2.reset(cstmt->executeQuery());
+  ASSERT(rs2->next());
+  ASSERT_EQUALS(2, rs2->getInt(1));
+  ASSERT(!rs2->next());
+  ASSERT(cstmt->getMoreResults());
+  ASSERT(rs2->isClosed());
+  res.reset(cstmt->getResultSet());
+  ASSERT(res->next());
+  ASSERT_EQUALS(1, res->getInt(1));
+  cstmt.reset();
+  ASSERT(res->isClosed());
+}
+
+
 /* Connector wasn't precise enough with double numbers operations and could lose significant digits */
 void preparedstatement::bugConcpp96()
 {
@@ -1964,7 +2008,12 @@ void preparedstatement::bugConcpp96()
  */
 void preparedstatement::concpp99_batchRewrite()
 {
-  sql::ConnectOptionsMap connection_properties{{"userName", user}, {"password", passwd}, {"rewriteBatchedStatements", "true"}, {"useTls", useTls ? "true" : "false"}};
+  sql::ConnectOptionsMap connection_properties{
+    {"userName", user},
+    {"password", passwd},
+    {"useBulkStmts", "false"},
+    {"rewriteBatchedStatements", "true"},
+    {"useTls", useTls ? "true" : "false"}};
 
   con.reset(driver->connect(url, connection_properties));
   // Reading results must be on the different connection to ensure that the driver commits the batch
@@ -2031,10 +2080,15 @@ void preparedstatement::concpp99_batchRewrite()
 
 
 /** Test of useBulkStmts option. The test does cannot test if the batch is really rewritten, though.
+ * Contains bits to test CONCPP-129 - introduced SQLString "nulliness"
  */
 void preparedstatement::concpp106_batchBulk()
 {
-  sql::ConnectOptionsMap connection_properties{ {"userName", user}, {"password", passwd}, {"useBulkStmts", "true"}, {"useTls", useTls ? "true" : "false"} };
+  // Testing that false(null) doesn't lead to option being treated as true.
+  sql::SQLString nullStr(nullptr);
+  ASSERT(nullStr == nullptr);
+  sql::ConnectOptionsMap connection_properties{ {"userName", user}, {"password", passwd},
+    {"useBulkStmts", "true"}, {"useTls", useTls ? "true" : nullptr} };
 
   con.reset(driver->connect(url, connection_properties));
   // Reading results must be on the different connection to ensure that the driver commits the batch
@@ -2049,11 +2103,10 @@ void preparedstatement::concpp106_batchBulk()
   const sql::SQLString selectQuery("SELECT id, val FROM concpp106_batchBulk ORDER BY id"),
     deleteQuery("DELETE FROM concpp106_batchBulk");
 
-  for (std::size_t i = 0; i < sizeof(insertQuery) / sizeof(insertQuery[0]); ++i) {
+  for (std::size_t i= 0; i < sizeof(insertQuery) / sizeof(insertQuery[0]); ++i) {
     pstmt.reset(con->prepareStatement(insertQuery[i]));
-    //ssps.reset(sspsCon->prepareStatement(insertQuery[i]));
 
-    for (uint32_t row = 0; row < sizeof(id) / sizeof(id[0]); ++row) {
+    for (uint32_t row= 0; row < sizeof(id) / sizeof(id[0]); ++row) {
       pstmt->setInt(1, id[row]);
       if (val[i][row] != nullptr) {
         pstmt->setString(2, val[0][row]);
@@ -2071,7 +2124,7 @@ void preparedstatement::concpp106_batchBulk()
 
     res.reset(stmt->executeQuery(selectQuery));
 
-    for (uint32_t row = 0; row < sizeof(id) / sizeof(id[0]); ++row) {
+    for (uint32_t row= 0; row < sizeof(id) / sizeof(id[0]); ++row) {
       ASSERT(res->next());
       ASSERT_EQUALS(id_expected[row], res->getInt(1));
       if (val_expected[i][row] == nullptr) {
@@ -2089,12 +2142,11 @@ void preparedstatement::concpp106_batchBulk()
     ASSERT(!res->next());
     ////// The same, but for executeLargeBatch
     stmt->executeUpdate(deleteQuery);
-    //const sql::Ints& batchRes2= ssps->executeBatch();
 
     pstmt->clearBatch();
     pstmt->clearParameters();
 
-    for (uint32_t row = 0; row < sizeof(id) / sizeof(id[0]); ++row) {
+    for (uint32_t row= 0; row < sizeof(id) / sizeof(id[0]); ++row) {
       pstmt->setInt(1, id[row] + 3);
       if (val[i][row] != nullptr) {
         pstmt->setString(2, val[0][row]);
@@ -2110,7 +2162,7 @@ void preparedstatement::concpp106_batchBulk()
     ASSERT_EQUALS(static_cast<uint64_t>(sizeof(id) / sizeof(id[0])), static_cast<uint64_t>(batchLRes.size()));
 
     res.reset(stmt->executeQuery(selectQuery));
-    for (uint32_t row = 0; row < sizeof(id) / sizeof(id[0]); ++row) {
+    for (uint32_t row= 0; row < sizeof(id) / sizeof(id[0]); ++row) {
       ASSERT(res->next());
       ASSERT_EQUALS(id_expected[row] + 3, res->getInt(1));
       if (val_expected[i][row] == nullptr) {
@@ -2132,6 +2184,76 @@ void preparedstatement::concpp106_batchBulk()
 }
 
 
+int32_t getPsCount(Connection& con)
+{
+  PreparedStatement ps(con->prepareStatement("SHOW GLOBAL STATUS LIKE 'Prepared_stmt_count'"));
+  ResultSet res(ps->executeQuery());
+  res->next();
+  return res->getInt(2);
+}
+
+/** Test of prepared statement cache. Requires exclusive server - otherwise may fail
+ */
+void preparedstatement::psCache()
+{
+  sql::ConnectOptionsMap connection_properties{{"userName", user},
+                                               {"password", passwd},
+                                               {"useServerPrepStmts", "true"},
+                                               {"cachePrepStmts", "true"},
+                                               {"prepStmtCacheSize", "2"},
+                                               {"prepStmtCacheSqlLimit", std::to_string(45 + 1 + db.length())},
+                                               {"useTls", useTls ? "true" : "false"}};
+
+  con.reset(driver->connect(url, connection_properties));
+  stmt.reset(con->createStatement());
+  createSchemaObject("TABLE", "psCache", "(id int not NULL PRIMARY KEY AUTO_INCREMENT, val VARCHAR(31))");
+
+  // getPsCount uses PS and thus - adds to cache and to total number of statements
+  int32_t initialCount= getPsCount(con);
+  pstmt.reset(con->prepareStatement("INSERT INTO psCache(val) values('XXX')"));
+
+  int32_t newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 1, newCount);
+
+  PreparedStatement ps2(con->prepareStatement("INSERT INTO psCache(val) values('XXX')"));
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 1, newCount);
+
+  ps2->execute();
+  pstmt.reset();
+
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 1,newCount);
+
+  // Should not get into the cache
+  pstmt.reset(con->prepareStatement("SELECT '12345648901234567890123456789012345 > 45'"));
+  ps2.reset(con->prepareStatement("SELECT '12345648901234567890123456789012345 > 45'"));
+
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 3, newCount);
+
+  // Should still be in cache
+  pstmt.reset(con->prepareStatement("INSERT INTO psCache(val) values('XXX')"));
+  // ... and one of previously prepared now closed
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 2, newCount);
+
+  // This should push other INSERT out of the cache, as it is full by now, and the query in the getPsCount
+  // was last requested from cache.
+  ps2.reset(con->prepareStatement("INSERT INTO psCache(val) values('0')"));
+  // Statement count is the same, as last SELECT PS is now closed, but pstmt is not closed, and holds reference to 1st INSERT PS
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 2, newCount);
+  pstmt.reset();
+  
+  newCount= getPsCount(con);
+  ASSERT_EQUALS(initialCount + 1, newCount);
+
+  // To make sure cache is cleared cleanly, and the framework provides next test with "standard" connection
+  con.reset();
+}
+
+
 void preparedstatement::concpp116_getByte()
 {
   pstmt.reset(sspsCon->prepareStatement("SELECT ?"));
@@ -2168,11 +2290,11 @@ void preparedstatement::concpp116_getByte()
 }
 
 /* CONCPP - 133 */
-void preparedstatement::multirs_caching()
+void preparedstatement::multirsCaching()
 {
   createSchemaObject("PROCEDURE", "ccpptest_multirs_caching", "()\
                         BEGIN\
-                          SELECT 1 as id, 'text' as val UNION SELECT 7 as id, 'seven' as val;\
+                          SELECT 17 as id, 'text' as val UNION SELECT 7 as id, 'seven' as val;\
                           SELECT 'some text';\
                           SELECT 2;\
                         END");
@@ -2185,7 +2307,7 @@ void preparedstatement::multirs_caching()
   /* Executing another query - stmt1 has to cache pending results */
   res.reset(stmt->executeQuery("SELECT 100"));
   /* Making sure we are at same position after caching */
-  ASSERT_EQUALS(1, res1->getInt(1));
+  ASSERT_EQUALS(17, res1->getInt(1));
   ASSERT_EQUALS("text", res1->getString("val"));
   ASSERT(res1->next());
   ASSERT_EQUALS(7, res1->getInt("id"));
@@ -2216,6 +2338,18 @@ void preparedstatement::multirs_caching()
   ASSERT_EQUALS(-1, pstmt1->getUpdateCount());
 }
 
+
+void preparedstatement::moreResultsAfterPrepare()
+{
+  pstmt.reset(con->prepareStatement("SELECT 1"));
+  ASSERT(!pstmt->getMoreResults());
+  ASSERT_EQUALS(-1, pstmt->getUpdateCount());
+
+  ssps.reset(con->prepareStatement("SELECT 1"));
+  ASSERT(!ssps->getMoreResults());
+  ASSERT_EQUALS(-1, ssps->getUpdateCount());
+}
+
 /* if sql::bytes wrapped C array, setBytes would crash the application */
 void preparedstatement::bytesArrParam()
 {
