@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
- *               2020, 2022 MariaDB Corporation AB
+ *               2020, 2024 MariaDB Corporation plc
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0, as
@@ -676,6 +676,24 @@ void statement::addBatch()
 }
 
 
+void statement::concpp88()
+{
+  res.reset(stmt->executeQuery("SELECT 1"));
+  ResultSet rs2(stmt->executeQuery("SELECT 2; SELECT 3"));
+
+  // Re-execution should close RS from previous execution
+  ASSERT(res->isClosed());
+  ASSERT(stmt->getMoreResults());
+
+  res.reset(stmt->getResultSet());
+  //Moving to the next RS should close previous RS
+  ASSERT(rs2->isClosed());
+
+  stmt.reset();
+  // Parent statemetne destruction should close RS
+  ASSERT(res->isClosed());
+}
+
 /** Test of rewriteBatchedStatements option. The test does cannot test if the batch is really rewritten, though.
  */
 void statement::concpp99_batchRewrite()
@@ -730,55 +748,6 @@ void statement::concpp99_batchRewrite()
   ASSERT_EQUALS(1LL, batchLRes[2]);
 }
 
-/* This is tmporary test for 1.0 version only. Thus putting all statement types in here - easier to remove */
-void statement::concpp107_setFetchSizeExeption()
-{
-  try {
-    stmt->setFetchSize(1);
-    FAIL("SQLFeatureNotImplementedException exception expected");
-  }
-  catch (sql::SQLFeatureNotImplementedException&)
-  {
-  }
-  
-  pstmt.reset(con->prepareStatement("SELECT 1 UNION SELECT 2"));
-  try {
-    pstmt->setFetchSize(1);
-    FAIL("SQLFeatureNotImplementedException exception expected");
-  }
-  catch (sql::SQLFeatureNotImplementedException&)
-  {
-    res.reset(pstmt->executeQuery());
-    ASSERT(res->next());
-    ASSERT(res->next());
-    ASSERT(!res->next());
-  }
-  res.reset(stmt->executeQuery("SELECT 1 UNION SELECT 2"));
-  ASSERT(res->next());
-  ASSERT(res->next());
-  ASSERT(!res->next());
-
-  stmt->setFetchSize(0);
-  res.reset(stmt->executeQuery("SELECT 1 UNION SELECT 2"));
-  ASSERT(res->next());
-  ASSERT(res->next());
-  ASSERT(!res->next());
-
-  createSchemaObject("PROCEDURE", "concpp100", "() BEGIN SELECT 1 UNION SELECT 2; END;");
-  cstmt.reset(con->prepareCall("CALL concpp100()"));
-  try {
-    cstmt->setFetchSize(1);
-    FAIL("SQLFeatureNotImplementedException exception expected");
-  }
-  catch (sql::SQLFeatureNotImplementedException&)
-  {
-    res.reset(cstmt->executeQuery());
-    ASSERT(res->next());
-    ASSERT(res->next());
-    ASSERT(!res->next());
-  }
-}
-
 /* CONCPP-132 getMoreResults highjacks other statement's pending result */
 void statement::otherstmts_result()
 {
@@ -835,5 +804,6 @@ void statement::multirs_caching()
   ASSERT(!stmt1->getMoreResults());
   ASSERT(stmt1->getUpdateCount() == -1);
 }
+
 } /* namespace statement */
 } /* namespace testsuite */
